@@ -29,14 +29,19 @@
 #include <utility>
 #include <vector>
 
-#include <boost/regex.hpp>
-
 #include <folly/FileUtil.h>
 #include <folly/MapUtil.h>
 #include <folly/String.h>
 #include <folly/container/Foreach.h>
 #include <folly/json.h>
 
+// This needs to be at the end because some versions end up including
+// Windows.h without defining NOMINMAX, which breaks uses
+// of `std::numeric_limits<T>::max()`. We explicitly define NOMINMAX here
+// explicitly instead.
+#define NOMINMAX 1
+#include <boost/regex.hpp>
+
 using namespace std;
 
 FOLLY_GFLAGS_DEFINE_bool(benchmark, false, "Run benchmarks.");
@@ -476,7 +481,7 @@ class BenchmarkResultsPrinter {
         // Print with baseline
         const auto rel = baselineNsPerIter_ / nsPerIter * 100.0;
         printf(
-            "%*s%7.5g%%  %9.9s  %8.8s",
+            "%*s%#7.5g%%  %9.9s  %8.8s",
             static_cast<int>(s.size()),
             s.c_str(),
             rel,
