@@ -15,31 +15,34 @@
  */
 
 /**
- *
- * This file provides a generic interface for converting objects to and from
+ * Conv provides the ubiquitous method `to<TargetType>(source)`, along with
+ * a few other generic interfaces for converting objects to and from
  * string-like types (std::string, fbstring, StringPiece), as well as
  * range-checked conversions between numeric and enum types. The mechanisms are
  * extensible, so that user-specified types can add folly::to support.
  *
+ *     folly::to<std::string>(123)
+ *     // "123"
+ *
  *******************************************************************************
- * TYPE -> STRING CONVERSIONS
+ * ## TYPE -> STRING CONVERSIONS
  *******************************************************************************
- * You can call the to<std::string> or to<fbstring>. These are variadic
+ * You can call the `to<std::string>` or `to<fbstring>`. These are variadic
  * functions that convert their arguments to strings, and concatenate them to
  * form a result. So, for example,
  *
- * auto str = to<std::string>(123, "456", 789);
+ *     auto str = to<std::string>(123, "456", 789);
  *
- * Sets str to "123456789".
+ * Sets str to `"123456789"`.
  *
  * In addition to just concatenating the arguments, related functions can
- * delimit them with some string: toDelim<std::string>(",", "123", 456, "789")
- * will return the string "123,456,789".
+ * delimit them with some string: `toDelim<std::string>(",", "123", 456, "789")`
+ * will return the string `"123,456,789"`.
  *
  * toAppend does not return a string; instead, it takes a pointer to a string as
  * its last argument, and appends the result of the concatenation into it:
- * std::string str = "123";
- * toAppend(456, "789", &str); // Now str is "123456789".
+ *     std::string str = "123";
+ *     toAppend(456, "789", &str); // Now str is "123456789".
  *
  * The toAppendFit function acts like toAppend, but it precalculates the size
  * required to perform the append operation, and reserves that space in the
@@ -52,49 +55,51 @@
  * and toAppendDelimFit are defined, with the obvious semantics.
  *
  *******************************************************************************
- * STRING -> TYPE CONVERSIONS
+ * ## STRING -> TYPE CONVERSIONS
  *******************************************************************************
  * Going in the other direction, and parsing a string into a C++ type, is also
  * supported:
- * to<int>("123"); // Returns 123.
+ *     to<int>("123"); // Returns 123.
  *
- * Out of range (e.g. to<std::uint8_t>("1000")), or invalidly formatted (e.g.
- * to<int>("four")) inputs will throw. If throw-on-error is undesirable (for
+ * Out of range (e.g. `to<std::uint8_t>("1000")`), or invalidly formatted (e.g.
+ * `to<int>("four")`) inputs will throw. If throw-on-error is undesirable (for
  * instance: you're dealing with untrusted input, and want to protect yourself
  * from users sending you down a very slow exception-throwing path), you can use
- * tryTo<T>, which will return an Expected<T, ConversionCode>.
+ * `tryTo<T>`, which will return an `Expected<T, ConversionCode>`.
  *
- * There are overloads of to() and tryTo() that take a StringPiece*. These parse
- * out a type from the beginning of a string, and modify the passed-in
+ * There are overloads of to() and tryTo() that take a `StringPiece*`. These
+ * parse out a type from the beginning of a string, and modify the passed-in
  * StringPiece to indicate the portion of the string not consumed.
  *
  *******************************************************************************
- * NUMERIC / ENUM CONVERSIONS
+ * ## NUMERIC / ENUM CONVERSIONS
  *******************************************************************************
- * Conv also supports a to<T>(S) overload, where T and S are numeric or enum
+ * Conv also supports a `to<T>(S)` overload, where T and S are numeric or enum
  * types, that checks to see that the target type can represent its argument,
- * and will throw if it cannot. This includes cases where a floating point ->
+ * and will throw if it cannot. This includes cases where a floating point to
  * integral conversion is attempted on a value with a non-zero fractional
- * component, and integral -> floating point conversions that would lose
+ * component, and integral to floating point conversions that would lose
  * precision. Enum conversions are range-checked for the underlying type of the
  * enum, but there is no check that the input value is a valid choice of enum
  * value.
  *
  *******************************************************************************
- * CUSTOM TYPE CONVERSIONS
+ * ## CUSTOM TYPE CONVERSIONS
  *******************************************************************************
  * Users may customize the string conversion functionality for their own data
- * types, . The key functions you should implement are:
- * // Two functions to allow conversion to your type from a string.
- * Expected<StringPiece, ConversionCode> parseTo(folly::StringPiece in,
- *     YourType& out);
- * YourErrorType makeConversionError(YourErrorType in, StringPiece in);
- * // Two functions to allow conversion from your type to a string.
- * template <class String>
- * void toAppend(const YourType& in, String* out);
- * size_t estimateSpaceNeeded(const YourType& in);
+ * types. The key functions you should implement are:
+ *     // Two functions to allow conversion to your type from a string.
+ *     Expected<StringPiece, ConversionCode> parseTo(folly::StringPiece in,
+ *         YourType& out);
+ *     YourErrorType makeConversionError(YourErrorType in, StringPiece in);
+ *     // Two functions to allow conversion from your type to a string.
+ *     template <class String>
+ *   void toAppend(const YourType& in, String* out);
+ *       size_t estimateSpaceNeeded(const YourType& in);
  *
  * These are documented below, inline.
+ *
+ * @file Conv.h
  */
 
 #pragma once
@@ -103,6 +108,7 @@
 #include <cassert>
 #include <cctype>
 #include <climits>
+#include <cmath>
 #include <cstddef>
 #include <limits>
 #include <stdexcept>
@@ -113,6 +119,7 @@
 
 #include <double-conversion/double-conversion.h> // V8 JavaScript implementation
 
+#include <folly/CPortability.h>
 #include <folly/Demangle.h>
 #include <folly/Expected.h>
 #include <folly/FBString.h>
@@ -148,11 +155,11 @@ enum class ConversionCode : unsigned char {
   NUM_ERROR_CODES, // has to be the last entry
 };
 
-struct ConversionErrorBase : std::range_error {
+struct FOLLY_EXPORT ConversionErrorBase : std::range_error {
   using std::range_error::range_error;
 };
 
-class ConversionError : public ConversionErrorBase {
+class FOLLY_EXPORT ConversionError : public ConversionErrorBase {
  public:
   ConversionError(const std::string& str, ConversionCode code)
       : ConversionErrorBase(str), code_(code) {}
@@ -166,7 +173,7 @@ class ConversionError : public ConversionErrorBase {
   ConversionCode code_;
 };
 
-/*******************************************************************************
+/**
  * Custom Error Translation
  *
  * Your overloaded parseTo() function can return a custom error code on failure.
@@ -186,7 +193,7 @@ class ConversionError : public ConversionErrorBase {
  *   ...
  *   return YourConversionError(messageString);
  * }
- ******************************************************************************/
+ */
 ConversionError makeConversionError(ConversionCode code, StringPiece input);
 
 namespace detail {
@@ -214,6 +221,8 @@ inline void enforceWhitespace(StringPiece sp) {
 } // namespace detail
 
 /**
+ * @overloadbrief to, but return an Expected
+ *
  * The identity conversion function.
  * tryTo<T>(T) returns itself for all types T.
  */
@@ -222,20 +231,23 @@ typename std::enable_if<
     std::is_same<Tgt, typename std::decay<Src>::type>::value,
     Expected<Tgt, ConversionCode>>::type
 tryTo(Src&& value) {
-  return std::forward<Src>(value);
+  return static_cast<Src&&>(value);
 }
 
+/**
+ * @overloadbrief Convert from one type to another.
+ */
 template <class Tgt, class Src>
 typename std::enable_if<
     std::is_same<Tgt, typename std::decay<Src>::type>::value,
     Tgt>::type
 to(Src&& value) {
-  return std::forward<Src>(value);
+  return static_cast<Src&&>(value);
 }
 
-/*******************************************************************************
+/**
  * Arithmetic to boolean
- ******************************************************************************/
+ */
 
 /**
  * Unchecked conversion from arithmetic to boolean. This is different from the
@@ -260,12 +272,15 @@ to(const Src& value) {
   return value != Src();
 }
 
-/*******************************************************************************
+/**
  * Anything to string
- ******************************************************************************/
+ */
 
 namespace detail {
 
+template <class... T>
+using LastElement = type_pack_element_t<sizeof...(T) - 1, T...>;
+
 #ifdef _MSC_VER
 // MSVC can't quite figure out the LastElementImpl::call() stuff
 // in the base implementation, so we have to use tuples instead,
@@ -273,55 +288,37 @@ namespace detail {
 // though the runtime performance is the same.
 
 template <typename... Ts>
-auto getLastElement(Ts&&... ts) -> decltype(std::get<sizeof...(Ts) - 1>(
-    std::forward_as_tuple(std::forward<Ts>(ts)...))) {
-  return std::get<sizeof...(Ts) - 1>(
-      std::forward_as_tuple(std::forward<Ts>(ts)...));
+const LastElement<Ts...>& getLastElement(const Ts&... ts) {
+  return std::get<sizeof...(Ts) - 1>(std::forward_as_tuple(ts...));
 }
 
 inline void getLastElement() {}
-
-template <size_t size, typename... Ts>
-struct LastElementType : std::tuple_element<size - 1, std::tuple<Ts...>> {};
-
-template <>
-struct LastElementType<0> {
-  using type = void;
-};
-
-template <class... Ts>
-struct LastElement
-    : std::decay<typename LastElementType<sizeof...(Ts), Ts...>::type> {};
 #else
-template <typename... Ts>
-struct LastElementImpl {
-  static void call(Ignored<Ts>...) {}
+template <typename...>
+struct LastElementImpl;
+template <>
+struct LastElementImpl<> {
+  static void call() {}
 };
-
-template <typename Head, typename... Ts>
-struct LastElementImpl<Head, Ts...> {
+template <typename Ign, typename... Igns>
+struct LastElementImpl<Ign, Igns...> {
   template <typename Last>
-  static Last call(Ignored<Ts>..., Last&& last) {
-    return std::forward<Last>(last);
+  static const Last& call(Igns..., const Last& last) {
+    return last;
   }
 };
 
 template <typename... Ts>
-auto getLastElement(const Ts&... ts)
-    -> decltype(LastElementImpl<Ts...>::call(ts...)) {
-  return LastElementImpl<Ts...>::call(ts...);
+const LastElement<Ts...>& getLastElement(const Ts&... ts) {
+  return LastElementImpl<Ignored<Ts>...>::call(ts...);
 }
-
-template <class... Ts>
-struct LastElement : std::decay<decltype(LastElementImpl<Ts...>::call(
-                         std::declval<Ts>()...))> {};
 #endif
 
 } // namespace detail
 
-/*******************************************************************************
+/**
  * Conversions from integral types to string types.
- ******************************************************************************/
+ */
 
 #if FOLLY_HAVE_INT128_T
 namespace detail {
@@ -336,31 +333,50 @@ constexpr unsigned int digitsEnough() {
   return static_cast<unsigned int>(digits10) + 1;
 }
 
-inline size_t unsafeTelescope128(
-    char* buffer, size_t room, unsigned __int128 x) {
-  typedef unsigned __int128 Usrc;
-  size_t p = room - 1;
+inline size_t unsafeTelescope128(char* outb, char* oute, unsigned __int128 x) {
+  using Usrc = unsigned __int128;
 
-  while (x >= (Usrc(1) << 64)) { // Using 128-bit division while needed
-    const auto y = x / 10;
-    const auto digit = x % 10;
+  // Decompose the input into at most 3 components using the largest power-of-10
+  // base that fits in a 64-bit unsigned integer, and then convert the
+  // components using 64-bit arithmetic and concatenate them.
+  constexpr static auto kBase = UINT64_C(10'000'000'000'000'000'000);
+  constexpr static size_t kBaseDigits = 19;
 
-    buffer[p--] = static_cast<char>('0' + digit);
-    x = y;
+  size_t p = 0;
+  const auto leading = [&](Usrc v) {
+    assert(v >> 64 == 0);
+    p = detail::to_ascii_with_route<10, to_ascii_alphabet_lower>(
+        outb, oute, static_cast<uint64_t>(v));
+  };
+  const auto append = [&](uint64_t v) {
+    assert(v < kBase);
+    assert(outb + p + kBaseDigits <= oute);
+    auto v64 = static_cast<uint64_t>(v);
+    detail::to_ascii_with_route<10, to_ascii_alphabet_lower>(
+        outb + p, kBaseDigits, v64);
+    p += kBaseDigits;
+  };
+
+  if (x >> 64 > 0) {
+    const auto rem = static_cast<uint64_t>(x % kBase);
+    x /= kBase;
+
+    if (x >> 64 > 0) {
+      const auto rem2 = static_cast<uint64_t>(x % kBase);
+      x /= kBase;
+
+      leading(x);
+      append(rem2);
+      append(rem);
+      return p;
+    }
+
+    leading(x);
+    append(rem);
+    return p;
   }
 
-  uint64_t xx = static_cast<uint64_t>(x); // Rest uses faster 64-bit division
-
-  while (xx >= 10) {
-    const auto y = xx / 10ULL;
-    const auto digit = xx % 10ULL;
-
-    buffer[p--] = static_cast<char>('0' + digit);
-    xx = y;
-  }
-
-  buffer[p] = static_cast<char>('0' + xx);
-
+  leading(x);
   return p;
 }
 
@@ -368,6 +384,8 @@ inline size_t unsafeTelescope128(
 #endif
 
 /**
+ * @overloadbrief Appends conversion to string.
+ *
  * A single char gets appended.
  */
 template <class Tgt>
@@ -375,6 +393,10 @@ void toAppend(char value, Tgt* result) {
   *result += value;
 }
 
+/**
+ * @overloadbrief Estimates the number of characters in a value's string
+ * representation.
+ */
 template <class T>
 constexpr typename std::enable_if<std::is_same<T, char>::value, size_t>::type
 estimateSpaceNeeded(T) {
@@ -481,26 +503,24 @@ template <class Tgt>
 void toAppend(__int128 value, Tgt* result) {
   typedef unsigned __int128 Usrc;
   char buffer[detail::digitsEnough<unsigned __int128>() + 1];
+  const auto oute = buffer + sizeof(buffer);
   size_t p;
 
   if (value < 0) {
-    p = detail::unsafeTelescope128(buffer, sizeof(buffer), -Usrc(value));
-    buffer[--p] = '-';
+    buffer[0] = '-';
+    p = 1 + detail::unsafeTelescope128(buffer + 1, oute, -Usrc(value));
   } else {
-    p = detail::unsafeTelescope128(buffer, sizeof(buffer), value);
+    p = detail::unsafeTelescope128(buffer, oute, value);
   }
 
-  result->append(buffer + p, buffer + sizeof(buffer));
+  result->append(buffer, p);
 }
 
 template <class Tgt>
 void toAppend(unsigned __int128 value, Tgt* result) {
   char buffer[detail::digitsEnough<unsigned __int128>()];
-  size_t p;
-
-  p = detail::unsafeTelescope128(buffer, sizeof(buffer), value);
-
-  result->append(buffer + p, buffer + sizeof(buffer));
+  size_t p = detail::unsafeTelescope128(buffer, buffer + sizeof(buffer), value);
+  result->append(buffer, p);
 }
 
 template <class T>
@@ -612,16 +632,16 @@ estimateSpaceNeeded(Src value) {
   return estimateSpaceNeeded(to_underlying(value));
 }
 
-/*******************************************************************************
+/**
  * Conversions from floating-point types to string types.
- ******************************************************************************/
+ */
 
 namespace detail {
 constexpr int kConvMaxDecimalInShortestLow = -6;
 constexpr int kConvMaxDecimalInShortestHigh = 21;
 } // namespace detail
 
-/** Wrapper around DoubleToStringConverter **/
+/** Wrapper around DoubleToStringConverter */
 template <class Tgt, class Src>
 typename std::enable_if<
     std::is_floating_point<Src>::value && IsSomeString<Tgt>::value>::type
@@ -629,10 +649,12 @@ toAppend(
     Src value,
     Tgt* result,
     double_conversion::DoubleToStringConverter::DtoaMode mode,
-    unsigned int numDigits) {
+    unsigned int numDigits,
+    double_conversion::DoubleToStringConverter::Flags flags =
+        double_conversion::DoubleToStringConverter::NO_FLAGS) {
   using namespace double_conversion;
   DoubleToStringConverter conv(
-      DoubleToStringConverter::NO_FLAGS,
+      flags,
       "Infinity",
       "NaN",
       'E',
@@ -731,73 +753,126 @@ estimateSpaceNeeded(const Src&) {
   return sizeof(Src) + 1; // dumbest best effort ever?
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 namespace detail {
 
-template <class Tgt>
-typename std::enable_if<IsSomeString<Tgt>::value, size_t>::type
-estimateSpaceToReserve(size_t sofar, Tgt*) {
-  return sofar;
+FOLLY_ERASE constexpr size_t estimateSpaceToReserveOne(std::false_type, void*) {
+  return 0;
+}
+template <typename T>
+FOLLY_ERASE constexpr size_t estimateSpaceToReserveOne(
+    std::true_type, const T& v) {
+  return estimateSpaceNeeded(v);
 }
 
-template <class T, class... Ts>
-size_t estimateSpaceToReserve(size_t sofar, const T& v, const Ts&... vs) {
-  return estimateSpaceToReserve(sofar + estimateSpaceNeeded(v), vs...);
-}
+template <typename>
+struct EstimateSpaceToReserveAll;
+template <size_t... I>
+struct EstimateSpaceToReserveAll<std::index_sequence<I...>> {
+  template <size_t J, size_t N = sizeof...(I)>
+  using tag = bool_constant<J + 1 < N>;
+  template <class... T>
+  static size_t call(const T&... v) {
+    const size_t sizes[] = {estimateSpaceToReserveOne(tag<I>{}, v)...};
+    size_t size = 0;
+    for (const auto s : sizes) {
+      size += s;
+    }
+    return size;
+  }
+};
 
-template <class... Ts>
-void reserveInTarget(const Ts&... vs) {
-  getLastElement(vs...)->reserve(estimateSpaceToReserve(0, vs...));
+template <class O>
+void reserveInTarget(const O& o) {
+  (void)o;
+}
+template <class T, class O>
+void reserveInTarget(const T& v, const O& o) {
+  o->reserve(estimateSpaceNeeded(v));
+}
+template <class T0, class T1, class... Ts>
+void reserveInTarget(const T0& v0, const T1& v1, const Ts&... vs) {
+  using seq = std::index_sequence_for<T0, T1, Ts...>;
+  getLastElement(vs...)->reserve(
+      EstimateSpaceToReserveAll<seq>::call(v0, v1, vs...));
 }
 
 template <class Delimiter, class... Ts>
 void reserveInTargetDelim(const Delimiter& d, const Ts&... vs) {
   static_assert(sizeof...(vs) >= 2, "Needs at least 2 args");
-  size_t fordelim = (sizeof...(vs) - 2) *
-      estimateSpaceToReserve(0, d, static_cast<std::string*>(nullptr));
-  getLastElement(vs...)->reserve(estimateSpaceToReserve(fordelim, vs...));
+  using seq = std::index_sequence_for<Ts...>;
+  size_t fordelim = (sizeof...(vs) - 2) * estimateSpaceNeeded(d);
+  getLastElement(vs...)->reserve(
+      fordelim + EstimateSpaceToReserveAll<seq>::call(vs...));
 }
 
-/**
- * Variadic base case: append one element
- */
+template <class T>
+FOLLY_ERASE constexpr int toAppendStrImplOne(
+    std::false_type, const T& v, void*) {
+  (void)v;
+  return 0;
+}
 template <class T, class Tgt>
-typename std::enable_if<
-    IsSomeString<typename std::remove_pointer<Tgt>::type>::value>::type
-toAppendStrImpl(const T& v, Tgt result) {
-  toAppend(v, result);
+FOLLY_ERASE int toAppendStrImplOne(std::true_type, const T& v, Tgt result) {
+  return toAppend(v, result), 0;
 }
+template <typename>
+struct ToAppendStrImplAll;
+template <size_t... I>
+struct ToAppendStrImplAll<std::index_sequence<I...>> {
+  template <class... T>
+  static void call(const T&... v) {
+    using _ = int[];
+    auto r = getLastElement(v...);
+    void(_{toAppendStrImplOne(bool_constant<I + 1 < sizeof...(T)>{}, v, r)...});
+  }
+};
 
-template <class T, class... Ts>
-typename std::enable_if<
-    sizeof...(Ts) >= 2 &&
-    IsSomeString<typename std::remove_pointer<
-        typename detail::LastElement<const Ts&...>::type>::type>::value>::type
-toAppendStrImpl(const T& v, const Ts&... vs) {
-  toAppend(v, getLastElement(vs...));
-  toAppendStrImpl(vs...);
+template <class Delimiter, class T>
+FOLLY_ERASE constexpr int toAppendDelimStrImplOne(
+    index_constant<0>, const Delimiter& d, const T& v, void*) {
+  (void)d;
+  (void)v;
+  return 0;
 }
-
 template <class Delimiter, class T, class Tgt>
-typename std::enable_if<
-    IsSomeString<typename std::remove_pointer<Tgt>::type>::value>::type
-toAppendDelimStrImpl(const Delimiter& /* delim */, const T& v, Tgt result) {
+FOLLY_ERASE int toAppendDelimStrImplOne(
+    index_constant<1>, const Delimiter& d, const T& v, Tgt result) {
+  (void)d;
   toAppend(v, result);
+  return 0;
 }
-
+template <class Delimiter, class T, class Tgt>
+FOLLY_ERASE int toAppendDelimStrImplOne(
+    index_constant<2>, const Delimiter& d, const T& v, Tgt result) {
+  toAppend(v, result);
+  toAppend(d, result);
+  return 0;
+}
+template <typename>
+struct ToAppendDelimStrImplAll;
+template <size_t... I>
+struct ToAppendDelimStrImplAll<std::index_sequence<I...>> {
+  template <size_t J, size_t N = sizeof...(I), size_t K = N - J - 1>
+  using tag = index_constant<(K < 2 ? K : 2)>;
+  template <class Delimiter, class... T>
+  static void call(const Delimiter& d, const T&... v) {
+    using _ = int[];
+    auto r = detail::getLastElement(v...);
+    void(_{toAppendDelimStrImplOne(tag<I>{}, d, v, r)...});
+  }
+};
 template <class Delimiter, class T, class... Ts>
 typename std::enable_if<
     sizeof...(Ts) >= 2 &&
     IsSomeString<typename std::remove_pointer<
-        typename detail::LastElement<const Ts&...>::type>::type>::value>::type
+        detail::LastElement<Ts...>>::type>::value>::type
 toAppendDelimStrImpl(const Delimiter& delim, const T& v, const Ts&... vs) {
-  // we are really careful here, calling toAppend with just one element does
-  // not try to estimate space needed (as we already did that). If we call
-  // toAppend(v, delim, ....) we would do unnecessary size calculation
-  toAppend(v, detail::getLastElement(vs...));
-  toAppend(delim, detail::getLastElement(vs...));
-  toAppendDelimStrImpl(delim, vs...);
+  using seq = std::index_sequence_for<T, Ts...>;
+  ToAppendDelimStrImplAll<seq>::call(delim, v, vs...);
 }
 } // namespace detail
+#endif
 
 /**
  * Variadic conversion to string. Appends each element in turn.
@@ -824,9 +899,10 @@ template <class... Ts>
 typename std::enable_if<
     sizeof...(Ts) >= 3 &&
     IsSomeString<typename std::remove_pointer<
-        typename detail::LastElement<const Ts&...>::type>::type>::value>::type
+        detail::LastElement<Ts...>>::type>::value>::type
 toAppend(const Ts&... vs) {
-  ::folly::detail::toAppendStrImpl(vs...);
+  using seq = std::index_sequence_for<Ts...>;
+  detail::ToAppendStrImplAll<seq>::call(vs...);
 }
 
 #ifdef _MSC_VER
@@ -841,6 +917,8 @@ void toAppend(const pid_t a, Tgt* res) {
 #endif
 
 /**
+ * @overloadbrief toAppend, but pre-allocate the exact amount of space required.
+ *
  * Special version of the call that preallocates exactly as much memory
  * as need for arguments to be stored in target. This means we are
  * not doing exponential growth when we append. If you are using it
@@ -851,7 +929,7 @@ void toAppend(const pid_t a, Tgt* res) {
  */
 template <class... Ts>
 typename std::enable_if<IsSomeString<typename std::remove_pointer<
-    typename detail::LastElement<const Ts&...>::type>::type>::value>::type
+    detail::LastElement<Ts...>>::type>::value>::type
 toAppendFit(const Ts&... vs) {
   ::folly::detail::reserveInTarget(vs...);
   toAppend(vs...);
@@ -868,6 +946,8 @@ typename std::enable_if<IsSomeString<Tgt>::value>::type toAppend(
     Tgt* /* result */) {}
 
 /**
+ * @overloadbrief Use a specified delimiter between appendees.
+ *
  * Variadic base case: do nothing.
  */
 template <class Delimiter, class Tgt>
@@ -891,17 +971,19 @@ template <class Delimiter, class... Ts>
 typename std::enable_if<
     sizeof...(Ts) >= 3 &&
     IsSomeString<typename std::remove_pointer<
-        typename detail::LastElement<const Ts&...>::type>::type>::value>::type
+        detail::LastElement<Ts...>>::type>::value>::type
 toAppendDelim(const Delimiter& delim, const Ts&... vs) {
   detail::toAppendDelimStrImpl(delim, vs...);
 }
 
 /**
+ * @overloadbrief toAppend with custom delimiter and exact pre-allocation.
+ *
  * Detail in comment for toAppendFit
  */
 template <class Delimiter, class... Ts>
 typename std::enable_if<IsSomeString<typename std::remove_pointer<
-    typename detail::LastElement<const Ts&...>::type>::type>::value>::type
+    detail::LastElement<Ts...>>::type>::value>::type
 toAppendDelimFit(const Delimiter& delim, const Ts&... vs) {
   detail::reserveInTargetDelim(delim, vs...);
   toAppendDelim(delim, vs...);
@@ -918,8 +1000,7 @@ template <class Tgt, class... Ts>
 typename std::enable_if<
     IsSomeString<Tgt>::value &&
         (sizeof...(Ts) != 1 ||
-         !std::is_same<Tgt, typename detail::LastElement<const Ts&...>::type>::
-             value),
+         !std::is_same<Tgt, detail::LastElement<void, Ts...>>::value),
     Tgt>::type
 to(const Ts&... vs) {
   Tgt result;
@@ -949,6 +1030,8 @@ to(Src value) {
 }
 
 /**
+ * @overloadbrief Like `to`, but uses a custom delimiter.
+ *
  * toDelim<SomeString>(SomeString str) returns itself.
  */
 template <class Tgt, class Delim, class Src>
@@ -957,7 +1040,7 @@ typename std::enable_if<
         std::is_same<Tgt, typename std::decay<Src>::type>::value,
     Tgt>::type
 toDelim(const Delim& /* delim */, Src&& value) {
-  return std::forward<Src>(value);
+  return static_cast<Src&&>(value);
 }
 
 /**
@@ -968,8 +1051,7 @@ template <class Tgt, class Delim, class... Ts>
 typename std::enable_if<
     IsSomeString<Tgt>::value &&
         (sizeof...(Ts) != 1 ||
-         !std::is_same<Tgt, typename detail::LastElement<const Ts&...>::type>::
-             value),
+         !std::is_same<Tgt, detail::LastElement<void, Ts...>>::value),
     Tgt>::type
 toDelim(const Delim& delim, const Ts&... vs) {
   Tgt result;
@@ -977,9 +1059,9 @@ toDelim(const Delim& delim, const Ts&... vs) {
   return result;
 }
 
-/*******************************************************************************
+/**
  * Conversions from string types to integral types.
- ******************************************************************************/
+ */
 
 namespace detail {
 
@@ -1109,16 +1191,14 @@ typename std::enable_if< //
     is_integral_v<Tgt> && !std::is_same<Tgt, bool>::value,
     Tgt>::type
 to(const char* b, const char* e) {
-  return tryTo<Tgt>(b, e).thenOrThrow(
-      [](Tgt res) { return res; },
-      [=](ConversionCode code) {
-        return makeConversionError(code, StringPiece(b, e));
-      });
+  return tryTo<Tgt>(b, e).thenOrThrow(identity, [=](ConversionCode code) {
+    return makeConversionError(code, StringPiece(b, e));
+  });
 }
 
-/*******************************************************************************
+/**
  * Conversions from string types to arithmetic types.
- ******************************************************************************/
+ */
 
 /**
  * Parsing strings to numeric types.
@@ -1132,9 +1212,9 @@ parseTo(StringPiece src, Tgt& out) {
       [&](Tgt res) { return void(out = res), src; });
 }
 
-/*******************************************************************************
+/**
  * Integral / Floating Point to integral / Floating Point
- ******************************************************************************/
+ */
 
 namespace detail {
 
@@ -1190,6 +1270,9 @@ typename std::enable_if<
         !std::is_same<Tgt, Src>::value,
     Expected<Tgt, ConversionCode>>::type
 convertTo(const Src& value) noexcept {
+  if (FOLLY_UNLIKELY(std::isinf(value))) {
+    return static_cast<Tgt>(value);
+  }
   if /* constexpr */ (
       std::numeric_limits<Tgt>::max() < std::numeric_limits<Src>::max()) {
     if (value > std::numeric_limits<Tgt>::max()) {
@@ -1214,8 +1297,10 @@ inline typename std::enable_if<
 checkConversion(const Src& value) {
   constexpr Src tgtMaxAsSrc = static_cast<Src>(std::numeric_limits<Tgt>::max());
   constexpr Src tgtMinAsSrc = static_cast<Src>(std::numeric_limits<Tgt>::min());
-  if (value >= tgtMaxAsSrc) {
-    if (value > tgtMaxAsSrc) {
+  // NOTE: The following two comparisons also handle the case where value is
+  // NaN, as all comparisons with NaN are false.
+  if (!(value < tgtMaxAsSrc)) {
+    if (!(value <= tgtMaxAsSrc)) {
       return false;
     }
     const Src mmax = folly::nextafter(tgtMaxAsSrc, Src());
@@ -1223,7 +1308,7 @@ checkConversion(const Src& value) {
         std::numeric_limits<Tgt>::max() - static_cast<Tgt>(mmax)) {
       return false;
     }
-  } else if (is_signed_v<Tgt> && value <= tgtMinAsSrc) {
+  } else if (value <= tgtMinAsSrc) {
     if (value < tgtMinAsSrc) {
       return false;
     }
@@ -1303,14 +1388,12 @@ tryTo(const Src& value) noexcept {
 template <typename Tgt, typename Src>
 typename std::enable_if<detail::IsArithToArith<Tgt, Src>::value, Tgt>::type to(
     const Src& value) {
-  return tryTo<Tgt>(value).thenOrThrow(
-      [](Tgt res) { return res; },
-      [&](ConversionCode e) {
-        return makeConversionError(e, detail::errorValue<Tgt>(value));
-      });
+  return tryTo<Tgt>(value).thenOrThrow(identity, [&](ConversionCode e) {
+    return makeConversionError(e, detail::errorValue<Tgt>(value));
+  });
 }
 
-/*******************************************************************************
+/**
  * Custom Conversions
  *
  * Any type can be used with folly::to by implementing parseTo. The
@@ -1321,7 +1404,7 @@ typename std::enable_if<detail::IsArithToArith<Tgt, Src>::value, Tgt>::type to(
  * ::folly::Expected<::folly::StringPiece, SomeErrorCode>
  *   parseTo(::folly::StringPiece, OtherType&) noexcept;
  * }
- ******************************************************************************/
+ */
 template <class T>
 FOLLY_NODISCARD typename std::enable_if<
     std::is_enum<T>::value,
@@ -1506,9 +1589,9 @@ Tgt to(StringPiece* src) {
           [=](Error e) { return makeConversionError(e, *src); });
 }
 
-/*******************************************************************************
+/**
  * Enum to anything and back
- ******************************************************************************/
+ */
 
 template <class Tgt, class Src>
 typename std::enable_if<
