@@ -20,7 +20,6 @@
 
 namespace folly {
 
-extern "C" void* __folly_memcpy(
-    void* __restrict dst, const void* __restrict src, std::size_t size);
+extern "C" void* __folly_memcpy(void* dst, const void* src, std::size_t size);
 
 } // namespace folly
