@@ -183,6 +183,14 @@ class IPAddress {
   static Expected<IPAddress, IPAddressFormatError> tryFromString(
       StringPiece str) noexcept;
 
+  /**
+   * Tries to create a new IPAddress instance from the provided sockaddr.
+   *
+   * On failure, returns IPAddressFormatError.
+   */
+  static Expected<IPAddress, IPAddressFormatError> tryFromSockAddr(
+      const sockaddr* addr) noexcept;
+
   /**
    * Create an IPAddress from a `uint32_t`, using network byte order.
    *
