@@ -167,7 +167,7 @@ const typename Map::mapped_type& get_ref_default(
  * key in the map, or nullptr if the key doesn't exist in the map.
  */
 template <class Map, typename Key = typename Map::key_type>
-const typename Map::mapped_type* get_ptr(const Map& map, const Key& key) {
+const auto* get_ptr(const Map& map, const Key& key) {
   auto pos = map.find(key);
   return (pos != map.end() ? &pos->second : nullptr);
 }
@@ -176,11 +176,41 @@ const typename Map::mapped_type* get_ptr(const Map& map, const Key& key) {
  * Non-const overload of the above.
  */
 template <class Map, typename Key = typename Map::key_type>
-typename Map::mapped_type* get_ptr(Map& map, const Key& key) {
+auto* get_ptr(Map& map, const Key& key) {
   auto pos = map.find(key);
   return (pos != map.end() ? &pos->second : nullptr);
 }
 
+/**
+ * Same as `get_ptr` but for `find` variants that search for two keys at once.
+ */
+template <class Map, typename Key = typename Map::key_type>
+std::pair<const typename Map::mapped_type*, const typename Map::mapped_type*>
+get_ptr2(const Map& map, const Key& key0, const Key& key1) {
+  const auto& iter_pair = map.find(key0, key1);
+  auto iter0 = iter_pair.first;
+  auto iter1 = iter_pair.second;
+  auto end = map.end();
+  return std::make_pair(
+      iter0 != end ? &iter0->second : nullptr,
+      iter1 != end ? &iter1->second : nullptr);
+}
+
+/**
+ * Same as `get_ptr` but for `find` variants that search for two keys at once.
+ */
+template <class Map, typename Key = typename Map::key_type>
+std::pair<typename Map::mapped_type*, typename Map::mapped_type*> get_ptr2(
+    Map& map, const Key& key0, const Key& key1) {
+  const auto& iter_pair = map.find(key0, key1);
+  auto iter0 = iter_pair.first;
+  auto iter1 = iter_pair.second;
+  auto end = map.end();
+  return std::make_pair(
+      iter0 != end ? &iter0->second : nullptr,
+      iter1 != end ? &iter1->second : nullptr);
+}
+
 // TODO: Remove the return type computations when clang 3.5 and gcc 5.1 are
 // the minimum supported versions.
 namespace detail {
