@@ -122,6 +122,7 @@
 
 #pragma once
 
+#include <folly/CancellationToken.h>
 #include <folly/Exception.h>
 #include <folly/Executor.h>
 #include <folly/Memory.h>
@@ -134,7 +135,6 @@
 #include <folly/lang/Exception.h>
 #include <folly/memory/SanitizeLeak.h>
 #include <folly/synchronization/Baton.h>
-#include <folly/synchronization/RWSpinLock.h>
 
 #include <algorithm>
 #include <atomic>
@@ -473,11 +473,17 @@ class SingletonVault {
 
   // Destroy all singletons; when complete, the vault can't create
   // singletons once again until reenableInstances() is called.
+  // If reenableInstances() will not be called, destroyInstancesFinal()
+  // should be used instead.
   void destroyInstances();
 
   // Enable re-creating singletons after destroyInstances() was called.
   void reenableInstances();
 
+  // Same as destroyInstances() but reenableInstances() should not be called
+  // after it. Starts a shutdown timer.
+  void destroyInstancesFinal();
+
   // For testing; how many registered and living singletons we have.
   size_t registeredSingletonCount() const {
     return singletons_.rlock()->size();
@@ -513,6 +519,15 @@ class SingletonVault {
     return &detail::createGlobal<SingletonVault, VaultTag>();
   }
 
+  // Get a cancellation token that gets triggered when singleton destruction
+  // starts.
+  //
+  // The underlying cancellation source gets reset when reenableInstances() is
+  // called.
+  folly::CancellationToken getDestructionCancellationToken() {
+    return cancellationSource_.wlock()->getToken();
+  }
+
   void setType(Type type) { type_.store(type, std::memory_order_relaxed); }
 
   void setShutdownTimeout(std::chrono::milliseconds shutdownTimeout) {
@@ -525,8 +540,6 @@ class SingletonVault {
 
   void addToShutdownLog(std::string message);
 
-  void startShutdownTimer();
-
   [[noreturn]] void fireShutdownTimer();
 
   void setFailOnUseAfterFork(bool failOnUseAfterFork) {
@@ -550,6 +563,8 @@ class SingletonVault {
   //    any of the singletons managed by folly::Singleton was requested.
   static void scheduleDestroyInstances();
 
+  void startShutdownTimer();
+
   typedef std::unordered_map<
       detail::TypeDescriptor,
       detail::SingletonHolderBase*,
@@ -586,6 +601,11 @@ class SingletonVault {
   std::atomic<bool> shutdownTimerStarted_{false};
   std::chrono::milliseconds shutdownTimeout_{std::chrono::minutes{5}};
   Synchronized<std::vector<std::string>> shutdownLog_;
+  // We use a lock around CancellationSource to get the guarantee that all
+  // cancellation callbacks that got triggered on requestCancellation() are done
+  // executing by the time we start destruction.  This prevents silent callbacks
+  // that take long to block destruction.
+  folly::Synchronized<CancellationSource> cancellationSource_;
   bool failOnUseAfterFork_{true};
 };
 
@@ -814,7 +834,7 @@ class LeakySingleton {
     auto& entry = entryInstance();
     std::lock_guard<std::mutex> lg(entry.mutex);
     if (entry.ptr) {
-      annotate_object_leaked(std::exchange(entry.ptr, nullptr));
+      lsan_ignore_object(std::atomic_exchange(&entry.ptr, (T*)nullptr));
     }
     entry.createFunc = createFunc;
     entry.state = State::Dead;
@@ -829,7 +849,7 @@ class LeakySingleton {
     Entry& operator=(const Entry&) = delete;
 
     std::atomic<State> state{State::NotRegistered};
-    T* ptr{nullptr};
+    std::atomic<T*> ptr{nullptr};
     CreateFunc createFunc;
     std::mutex mutex;
     detail::TypeDescriptor type_{typeid(T), typeid(Tag)};
