@@ -626,7 +626,7 @@ class SynchronizedBase<Subclass, detail::SynchronizedMutexLevel::Unique> {
  * @tparam T  The type of datum to be stored.
  * @tparam Mutex  The mutex type that guards the datum. Must be Lockable.
  *
- * @refcode docs/examples/folly/Synchronized.cpp
+ * @refcode folly/docs/examples/folly/Synchronized.cpp
  */
 template <class T, class Mutex = SharedMutex>
 struct Synchronized : public SynchronizedBase<
@@ -653,7 +653,7 @@ struct Synchronized : public SynchronizedBase<
   /**
    * Default constructor leaves both members call their own default constructor.
    */
-  Synchronized() = default;
+  constexpr Synchronized() = default;
 
  public:
   /**
@@ -704,7 +704,7 @@ struct Synchronized : public SynchronizedBase<
    * instance `in_place` as the first argument.
    */
   template <typename... Args>
-  explicit Synchronized(in_place_t, Args&&... args)
+  explicit constexpr Synchronized(in_place_t, Args&&... args)
       : datum_(std::forward<Args>(args)...) {}
 
   /**
@@ -1289,12 +1289,24 @@ class LockedPtr {
    * Move constructor.
    */
   LockedPtr(LockedPtr&& rhs) noexcept = default;
+  template <
+      typename Type = SynchronizedType,
+      std::enable_if_t<std::is_const<Type>::value, int> = 0>
+  /* implicit */ LockedPtr(LockedPtr<Synchronized, LockPolicy>&& rhs) noexcept
+      : lock_{std::move(rhs.lock_)} {}
   template <
       typename LockPolicyType,
       EnableIfSameLevel<LockPolicyType>* = nullptr>
   explicit LockedPtr(
       LockedPtr<SynchronizedType, LockPolicyType>&& other) noexcept
       : lock_{std::move(other.lock_)} {}
+  template <
+      typename Type = SynchronizedType,
+      typename LockPolicyType,
+      std::enable_if_t<std::is_const<Type>::value, int> = 0,
+      EnableIfSameLevel<LockPolicyType>* = nullptr>
+  explicit LockedPtr(LockedPtr<Synchronized, LockPolicyType>&& rhs) noexcept
+      : lock_{std::move(rhs.lock_)} {}
 
   /**
    * Move assignment operator.
@@ -1308,6 +1320,16 @@ class LockedPtr {
     lock_ = std::move(other.lock_);
     return *this;
   }
+  template <
+      typename Type = SynchronizedType,
+      typename LockPolicyType,
+      std::enable_if_t<std::is_const<Type>::value, int> = 0,
+      EnableIfSameLevel<LockPolicyType>* = nullptr>
+  LockedPtr& operator=(
+      LockedPtr<Synchronized, LockPolicyType>&& other) noexcept {
+    lock_ = std::move(other.lock_);
+    return *this;
+  }
 
   /*
    * Copy constructor and assignment operator are deleted.
@@ -1460,6 +1482,15 @@ class LockedPtr {
     return transition_to_shared_lock(lock_);
   }
 
+  SynchronizedType* parent() const {
+    using simulacrum = typename SynchronizedType::Simulacrum;
+    static_assert(sizeof(simulacrum) == sizeof(SynchronizedType), "mismatch");
+    static_assert(alignof(simulacrum) == alignof(SynchronizedType), "mismatch");
+    constexpr auto off = offsetof(simulacrum, mutex_);
+    const auto raw = reinterpret_cast<char*>(lock_.mutex());
+    return reinterpret_cast<SynchronizedType*>(raw - (raw ? off : 0));
+  }
+
  private:
   /* implicit */ LockedPtr(LockType lock) noexcept : lock_{std::move(lock)} {}
 
@@ -1494,15 +1525,6 @@ class LockedPtr {
     lock_ = doLock(parent->mutex_);
   }
 
-  SynchronizedType* parent() const {
-    using simulacrum = typename SynchronizedType::Simulacrum;
-    static_assert(sizeof(simulacrum) == sizeof(SynchronizedType), "mismatch");
-    static_assert(alignof(simulacrum) == alignof(SynchronizedType), "mismatch");
-    constexpr auto off = offsetof(simulacrum, mutex_);
-    const auto raw = reinterpret_cast<char*>(lock_.mutex());
-    return reinterpret_cast<SynchronizedType*>(raw - (raw ? off : 0));
-  }
-
   LockType lock_;
 };
 
