@@ -103,12 +103,12 @@ class ValueHolder<
 
     void acquireLink() {
       uint32_t count = numlinks_.fetch_add(1, std::memory_order_release);
-      DCHECK_GE(count, 1);
+      DCHECK_GE(count, 1u);
     }
 
     bool releaseLink() {
       uint32_t count = numlinks_.load(std::memory_order_acquire);
-      DCHECK_GE(count, 1);
+      DCHECK_GE(count, 1u);
       if (count > 1) {
         count = numlinks_.fetch_sub(1, std::memory_order_acq_rel);
       }
@@ -302,24 +302,26 @@ class alignas(64) BucketTable {
   template <typename MatchFunc, typename K, typename... Args>
   bool insert(
       Iterator& it,
+      size_t h,
       const K& k,
       InsertType type,
       MatchFunc match,
       hazptr_obj_cohort<Atom>* cohort,
       Args&&... args) {
     return doInsert(
-        it, k, type, match, nullptr, cohort, std::forward<Args>(args)...);
+        it, h, k, type, match, nullptr, cohort, std::forward<Args>(args)...);
   }
 
   template <typename MatchFunc, typename K, typename... Args>
   bool insert(
       Iterator& it,
+      size_t h,
       const K& k,
       InsertType type,
       MatchFunc match,
       Node* cur,
       hazptr_obj_cohort<Atom>* cohort) {
-    return doInsert(it, k, type, match, cur, cohort, cur);
+    return doInsert(it, h, k, type, match, cur, cohort, cur);
   }
 
   // Must hold lock.
@@ -383,10 +385,9 @@ class alignas(64) BucketTable {
   }
 
   template <typename K>
-  bool find(Iterator& res, const K& k) {
+  bool find(Iterator& res, size_t h, const K& k) {
     auto& hazcurr = res.hazptrs_[1];
     auto& haznext = res.hazptrs_[2];
-    auto h = HashFn()(k);
     size_t bcount;
     Buckets* buckets;
     getBucketsAndCount(bcount, buckets, res.hazptrs_[0]);
@@ -406,9 +407,8 @@ class alignas(64) BucketTable {
   }
 
   template <typename K, typename MatchFunc>
-  std::size_t erase(const K& key, Iterator* iter, MatchFunc match) {
+  std::size_t erase(size_t h, const K& key, Iterator* iter, MatchFunc match) {
     Node* node{nullptr};
-    auto h = HashFn()(key);
     {
       std::lock_guard<Mutex> g(m_);
 
@@ -658,13 +658,13 @@ class alignas(64) BucketTable {
   template <typename MatchFunc, typename K, typename... Args>
   bool doInsert(
       Iterator& it,
+      size_t h,
       const K& k,
       InsertType type,
       MatchFunc match,
       Node* cur,
       hazptr_obj_cohort<Atom>* cohort,
       Args&&... args) {
-    auto h = HashFn()(k);
     std::unique_lock<Mutex> g(m_);
 
     size_t bcount = bucket_count_.load(std::memory_order_relaxed);
@@ -892,7 +892,7 @@ class alignas(64) SIMDTable {
       std::size_t off = index % 8;
       Atom<uint64_t>& old_tags = off == index ? tags_low_ : tags_hi_;
       uint64_t new_tags = old_tags.load(std::memory_order_relaxed);
-      uint64_t mask = 0xffUL << (off * 8);
+      uint64_t mask = 0xffULL << (off * 8);
       new_tags = (new_tags & ~mask) | (tag << (off * 8));
       old_tags.store(new_tags, std::memory_order_release);
     }
@@ -947,9 +947,9 @@ class alignas(64) SIMDTable {
     Atom<Node*>& item(size_t i) { return *launder(itemAddr(i)); }
 
     static constexpr uint64_t kOutboundOverflowIndex = 7 * 8;
-    static constexpr uint64_t kSaturatedOutboundOverflowCount = 0xffUL
+    static constexpr uint64_t kSaturatedOutboundOverflowCount = 0xffULL
         << kOutboundOverflowIndex;
-    static constexpr uint64_t kOutboundOverflowOperand = 0x1UL
+    static constexpr uint64_t kOutboundOverflowOperand = 0x1ULL
         << kOutboundOverflowIndex;
 
     unsigned outboundOverflowCount() const {
@@ -974,7 +974,7 @@ class alignas(64) SIMDTable {
     }
 
     static constexpr uint64_t kHostedOverflowIndex = 6 * 8;
-    static constexpr uint64_t kHostedOverflowOperand = 0x10UL
+    static constexpr uint64_t kHostedOverflowOperand = 0x10ULL
         << kHostedOverflowIndex;
 
     unsigned hostedOverflowCount() const {
@@ -1206,6 +1206,7 @@ class alignas(64) SIMDTable {
   template <typename MatchFunc, typename K, typename... Args>
   bool insert(
       Iterator& it,
+      size_t h,
       const K& k,
       InsertType type,
       MatchFunc match,
@@ -1215,7 +1216,6 @@ class alignas(64) SIMDTable {
     Chunks* chunks;
     size_t ccount, chunk_idx, tag_idx;
 
-    auto h = HashFn()(k);
     auto hp = splitHash(h);
 
     std::unique_lock<Mutex> g(m_);
@@ -1260,6 +1260,7 @@ class alignas(64) SIMDTable {
   template <typename MatchFunc, typename K, typename... Args>
   bool insert(
       Iterator& it,
+      size_t h,
       const K& k,
       InsertType type,
       MatchFunc match,
@@ -1270,7 +1271,6 @@ class alignas(64) SIMDTable {
     Chunks* chunks;
     size_t ccount, chunk_idx, tag_idx;
 
-    auto h = HashFn()(k);
     auto hp = splitHash(h);
 
     std::unique_lock<Mutex> g(m_);
@@ -1315,9 +1315,8 @@ class alignas(64) SIMDTable {
   }
 
   template <typename K>
-  bool find(Iterator& res, const K& k) {
+  bool find(Iterator& res, size_t h, const K& k) {
     auto& hazz = res.hazptrs_[1];
-    auto h = HashFn()(k);
     auto hp = splitHash(h);
     size_t ccount;
     Chunks* chunks;
@@ -1348,8 +1347,7 @@ class alignas(64) SIMDTable {
   }
 
   template <typename K, typename MatchFunc>
-  std::size_t erase(const K& key, Iterator* iter, MatchFunc match) {
-    auto h = HashFn()(key);
+  std::size_t erase(size_t h, const K& key, Iterator* iter, MatchFunc match) {
     const HashPair hp = splitHash(h);
 
     std::unique_lock<Mutex> g(m_);
@@ -1717,16 +1715,17 @@ class alignas(64) ConcurrentHashMapSegment {
   bool empty() { return impl_.empty(); }
 
   template <typename Key>
-  bool insert(Iterator& it, std::pair<Key, mapped_type>&& foo) {
-    return insert(it, std::move(foo.first), std::move(foo.second));
+  bool insert(Iterator& it, size_t h, std::pair<Key, mapped_type>&& foo) {
+    return insert(it, h, std::move(foo.first), std::move(foo.second));
   }
 
   template <typename Key, typename Value>
-  bool insert(Iterator& it, Key&& k, Value&& v) {
+  bool insert(Iterator& it, size_t h, Key&& k, Value&& v) {
     auto node = (Node*)Allocator().allocate(sizeof(Node));
     new (node) Node(cohort_, std::forward<Key>(k), std::forward<Value>(v));
     auto res = insert_internal(
         it,
+        h,
         node->getItem().first,
         InsertType::DOES_NOT_EXIST,
         [](const ValueType&) { return false; },
@@ -1739,11 +1738,12 @@ class alignas(64) ConcurrentHashMapSegment {
   }
 
   template <typename Key, typename... Args>
-  bool try_emplace(Iterator& it, Key&& k, Args&&... args) {
+  bool try_emplace(Iterator& it, size_t h, Key&& k, Args&&... args) {
     // Note: first key is only ever compared.  Second is moved in to
     // create the node, and the first key is never touched again.
     return insert_internal(
         it,
+        h,
         std::forward<Key>(k),
         InsertType::DOES_NOT_EXIST,
         [](const ValueType&) { return false; },
@@ -1752,9 +1752,10 @@ class alignas(64) ConcurrentHashMapSegment {
   }
 
   template <typename... Args>
-  bool emplace(Iterator& it, const KeyType& k, Node* node) {
+  bool emplace(Iterator& it, size_t h, const KeyType& k, Node* node) {
     return insert_internal(
         it,
+        h,
         k,
         InsertType::DOES_NOT_EXIST,
         [](const ValueType&) { return false; },
@@ -1762,11 +1763,12 @@ class alignas(64) ConcurrentHashMapSegment {
   }
 
   template <typename Key, typename Value>
-  bool insert_or_assign(Iterator& it, Key&& k, Value&& v) {
+  bool insert_or_assign(Iterator& it, size_t h, Key&& k, Value&& v) {
     auto node = (Node*)Allocator().allocate(sizeof(Node));
     new (node) Node(cohort_, std::forward<Key>(k), std::forward<Value>(v));
     auto res = insert_internal(
         it,
+        h,
         node->getItem().first,
         InsertType::ANY,
         [](const ValueType&) { return false; },
@@ -1779,11 +1781,12 @@ class alignas(64) ConcurrentHashMapSegment {
   }
 
   template <typename Key, typename Value>
-  bool assign(Iterator& it, Key&& k, Value&& v) {
+  bool assign(Iterator& it, size_t h, Key&& k, Value&& v) {
     auto node = (Node*)Allocator().allocate(sizeof(Node));
     new (node) Node(cohort_, std::forward<Key>(k), std::forward<Value>(v));
     auto res = insert_internal(
         it,
+        h,
         node->getItem().first,
         InsertType::MUST_EXIST,
         [](const ValueType&) { return false; },
@@ -1796,12 +1799,13 @@ class alignas(64) ConcurrentHashMapSegment {
   }
   template <typename Key, typename Value, typename Predicate>
   bool assign_if(
-      Iterator& it, Key&& k, Value&& desired, Predicate&& predicate) {
+      Iterator& it, size_t h, Key&& k, Value&& desired, Predicate&& predicate) {
     auto node = (Node*)Allocator().allocate(sizeof(Node));
     new (node)
         Node(cohort_, std::forward<Key>(k), std::forward<Value>(desired));
     auto res = insert_internal(
         it,
+        h,
         node->getItem().first,
         InsertType::MATCH,
         std::forward<Predicate>(predicate),
@@ -1815,9 +1819,14 @@ class alignas(64) ConcurrentHashMapSegment {
 
   template <typename Key, typename Value>
   bool assign_if_equal(
-      Iterator& it, Key&& k, const ValueType& expected, Value&& desired) {
+      Iterator& it,
+      size_t h,
+      Key&& k,
+      const ValueType& expected,
+      Value&& desired) {
     return assign_if(
         it,
+        h,
         std::forward<Key>(k),
         std::forward<Value>(desired),
         [&expected](const ValueType& v) { return v == expected; });
@@ -1826,18 +1835,24 @@ class alignas(64) ConcurrentHashMapSegment {
   template <typename MatchFunc, typename K, typename... Args>
   bool insert_internal(
       Iterator& it,
+      size_t h,
       const K& k,
       InsertType type,
       MatchFunc match,
       Args&&... args) {
     return impl_.insert(
-        it, k, type, match, cohort_, std::forward<Args>(args)...);
+        it, h, k, type, match, cohort_, std::forward<Args>(args)...);
   }
 
   template <typename MatchFunc, typename K, typename... Args>
   bool insert_internal(
-      Iterator& it, const K& k, InsertType type, MatchFunc match, Node* cur) {
-    return impl_.insert(it, k, type, match, cur, cohort_);
+      Iterator& it,
+      size_t h,
+      const K& k,
+      InsertType type,
+      MatchFunc match,
+      Node* cur) {
+    return impl_.insert(it, h, k, type, match, cur, cohort_);
   }
 
   // Must hold lock.
@@ -1846,24 +1861,26 @@ class alignas(64) ConcurrentHashMapSegment {
   }
 
   template <typename K>
-  bool find(Iterator& res, const K& k) {
-    return impl_.find(res, k);
+  bool find(Iterator& res, size_t h, const K& k) {
+    return impl_.find(res, h, k);
   }
 
   // Listed separately because we need a prev pointer.
   template <typename K>
-  size_type erase(const K& key) {
-    return erase_internal(key, nullptr, [](const ValueType&) { return true; });
+  size_type erase(size_t h, const K& key) {
+    return erase_internal(
+        h, key, nullptr, [](const ValueType&) { return true; });
   }
 
   template <typename K, typename Predicate>
-  size_type erase_key_if(const K& key, Predicate&& predicate) {
-    return erase_internal(key, nullptr, std::forward<Predicate>(predicate));
+  size_type erase_key_if(size_t h, const K& key, Predicate&& predicate) {
+    return erase_internal(h, key, nullptr, std::forward<Predicate>(predicate));
   }
 
   template <typename K, typename MatchFunc>
-  size_type erase_internal(const K& key, Iterator* iter, MatchFunc match) {
-    return impl_.erase(key, iter, match);
+  size_type erase_internal(
+      size_t h, const K& key, Iterator* iter, MatchFunc match) {
+    return impl_.erase(h, key, iter, match);
   }
 
   // Unfortunately because we are reusing nodes on rehash, we can't
@@ -1872,8 +1889,8 @@ class alignas(64) ConcurrentHashMapSegment {
   //
   // This is a small departure from standard stl containers: erase may
   // throw if hash or key_eq functions throw.
-  void erase(Iterator& res, Iterator& pos) {
-    erase_internal(pos->first, &res, [](const ValueType&) { return true; });
+  void erase(Iterator& res, Iterator& pos, size_t h) {
+    erase_internal(h, pos->first, &res, [](const ValueType&) { return true; });
     // Invalidate the iterator.
     pos = cend();
   }
