@@ -22,6 +22,7 @@
 
 #include <folly/portability/GTest.h>
 #include <folly/portability/SysResource.h>
+#include <folly/test/TestUtils.h>
 
 #include <glog/logging.h>
 
@@ -1034,8 +1035,11 @@ TEST(CacheLocality, BenchmarkSysfs) {
 
 #if defined(FOLLY_HAVE_LINUX_VDSO) && !defined(FOLLY_SANITIZE_MEMORY)
 TEST(Getcpu, VdsoGetcpu) {
+  Getcpu::Func func = Getcpu::resolveVdsoFunc();
+  SKIP_IF(func == nullptr);
+
   unsigned cpu;
-  Getcpu::resolveVdsoFunc()(&cpu, nullptr, nullptr);
+  func(&cpu, nullptr, nullptr);
 
   EXPECT_TRUE(cpu < CPU_SETSIZE);
 }
@@ -1150,11 +1154,13 @@ TEST(CoreAllocator, Basic) {
   coreFree(res);
 
   res = coreMalloc(8, kNumStripes, 0);
-  EXPECT_TRUE((intptr_t)res % 8 == 0); // check alignment
+  EXPECT_EQ(0, (intptr_t)res % 8); // check alignment
   memset(res, 0, 8);
   coreFree(res);
   res = coreMalloc(12, kNumStripes, 0);
-  EXPECT_TRUE((intptr_t)res % 16 == 0); // check alignment
+  if (alignof(std::max_align_t) >= 16) {
+    EXPECT_EQ(0, (intptr_t)res % 16); // check alignment
+  }
   memset(res, 0, 12);
   coreFree(res);
   res = coreMalloc(257, kNumStripes, 0);
