@@ -253,22 +253,6 @@ FOLLY_ALWAYS_INLINE static void prefetchAddr(T const* ptr) {
 #endif
 }
 
-/// Prefetch the object at ptr, starting at the cacheLineOffset cache line,
-/// and prefetching at most maxCacheLines cache lines.
-template <typename T>
-FOLLY_ALWAYS_INLINE static void prefetchAddr(
-    T const* ptr, size_t cacheLineOffset, size_t maxCacheLines) {
-  size_t constexpr kCacheLineSize = hardware_constructive_interference_size;
-  auto constexpr kObjectCacheLines =
-      (sizeof(T) + kCacheLineSize - 1) / kCacheLineSize;
-  size_t const cacheLines = std::min(kObjectCacheLines, maxCacheLines);
-
-  auto const bytes = static_cast<char const*>(static_cast<void const*>(ptr));
-  for (size_t i = cacheLineOffset; i < cacheLines; ++i) {
-    prefetchAddr(bytes + i * kCacheLineSize);
-  }
-}
-
 #if FOLLY_NEON
 using TagVector = uint8x16_t;
 #else // SSE2
@@ -1486,35 +1470,20 @@ class F14Table : public Policy {
   }
 
  public:
-  // Prehashing splits the work of find(key) into two calls, enabling you
-  // to manually implement loop pipelining for hot bulk lookups.  prehash
-  // computes the hash and prefetches the first computed memory location,
-  // and the two-arg find(F14HashToken,K) performs the rest of the search.
+  // prehash()/prefetch() split the work of find(key) into three calls, enabling
+  // you to manually implement loop pipelining for hot bulk lookups. prehash()
+  // computes the hash and prefetch() prefetches the first computed memory
+  // location, and the two-arg find(F14HashToken, K) performs the rest of the
+  // search.
   template <typename K>
   F14HashToken prehash(K const& key) const {
-    FOLLY_SAFE_DCHECK(chunks_ != nullptr, "");
-    auto hp = splitHash(this->computeKeyHash(key));
-    ChunkPtr firstChunk = chunks_ + moduloByChunkCount(hp.first);
-    prefetchAddr(firstChunk);
-    return F14HashToken(std::move(hp));
+    return F14HashToken{splitHash(this->computeKeyHash(key))};
   }
 
-  // prefetch() fetches the next two cache lines of the chunk, up to the
-  // chunk size. This is not included in prehash() because these loads are
-  // speculative. find() will always need to load the first cache line of
-  // the chunk, but it won't always need to load further cache lines. If
-  // they aren't needed, then prefetching them will hurt application
-  // performance, by polluting the local CPU cache. So prefetch() should
-  // only be called when benchmarks show performance improvements.
   void prefetch(F14HashToken const& token) const {
-    auto hp = static_cast<HashPair>(token);
-    ChunkPtr firstChunk = chunks_ + moduloByChunkCount(hp.first);
-    // The first cache line was already prefetched in prehash().
-    // Prefetch the next two cache lines up to the chunk size. We only
-    // fetch at most 2 more cache lines to avoid thrashing the local
-    // CPU cache when the chunk size is large. The item we're looking for
-    // is more likely to be near the beginning of the chunk.
-    prefetchAddr(firstChunk, /* offset */ 1, /* maxCacheLines */ 3);
+    FOLLY_SAFE_DCHECK(chunks_ != nullptr, "");
+    ChunkPtr firstChunk = chunks_ + moduloByChunkCount(token.hp_.first);
+    prefetchAddr(firstChunk);
   }
 
   template <typename K>
