@@ -18,6 +18,7 @@
 
 #include <folly/Portability.h>
 #include <folly/executors/IOExecutor.h>
+#include <folly/executors/QueueObserver.h>
 #include <folly/executors/ThreadPoolExecutor.h>
 #include <folly/io/async/EventBaseManager.h>
 #include <folly/synchronization/RelaxedAtomic.h>
@@ -56,12 +57,28 @@ FOLLY_MSVC_DISABLE_WARNING(4250)
  */
 class IOThreadPoolExecutor : public ThreadPoolExecutor, public IOExecutor {
  public:
+  struct Options {
+    Options() : waitForAll(false), enableThreadIdCollection(false) {}
+
+    Options& setWaitForAll(bool b) {
+      this->waitForAll = b;
+      return *this;
+    }
+    Options& setEnableThreadIdCollection(bool b) {
+      this->enableThreadIdCollection = b;
+      return *this;
+    }
+
+    bool waitForAll;
+    bool enableThreadIdCollection;
+  };
+
   explicit IOThreadPoolExecutor(
       size_t numThreads,
       std::shared_ptr<ThreadFactory> threadFactory =
           std::make_shared<NamedThreadFactory>("IOThreadPool"),
       folly::EventBaseManager* ebm = folly::EventBaseManager::get(),
-      bool waitForAll = false);
+      Options options = Options());
 
   IOThreadPoolExecutor(
       size_t maxThreads,
@@ -69,7 +86,7 @@ class IOThreadPoolExecutor : public ThreadPoolExecutor, public IOExecutor {
       std::shared_ptr<ThreadFactory> threadFactory =
           std::make_shared<NamedThreadFactory>("IOThreadPool"),
       folly::EventBaseManager* ebm = folly::EventBaseManager::get(),
-      bool waitForAll = false);
+      Options options = Options());
 
   ~IOThreadPoolExecutor() override;
 
@@ -81,6 +98,10 @@ class IOThreadPoolExecutor : public ThreadPoolExecutor, public IOExecutor {
 
   folly::EventBase* getEventBase() override;
 
+  // Ensures that the maximum number of active threads is running and returns
+  // the EventBase associated with each thread.
+  std::vector<folly::Executor::KeepAlive<folly::EventBase>> getAllEventBases();
+
   static folly::EventBase* getEventBase(ThreadPoolExecutor::ThreadHandle*);
 
   static std::mutex* getEventBaseShutdownMutex(
@@ -88,6 +109,11 @@ class IOThreadPoolExecutor : public ThreadPoolExecutor, public IOExecutor {
 
   folly::EventBaseManager* getEventBaseManager();
 
+  // Returns nullptr unless explicitly enabled through constructor
+  folly::WorkerProvider* getThreadIdCollector() {
+    return threadIdCollector_.get();
+  }
+
  protected:
   struct alignas(Thread) IOThread : public Thread {
     IOThread(IOThreadPoolExecutor* pool)
@@ -109,6 +135,7 @@ class IOThreadPoolExecutor : public ThreadPoolExecutor, public IOExecutor {
   relaxed_atomic<size_t> nextThread_;
   folly::ThreadLocal<std::shared_ptr<IOThread>> thisThread_;
   folly::EventBaseManager* eventBaseManager_;
+  std::unique_ptr<ThreadIdWorkerProvider> threadIdCollector_;
 };
 
 FOLLY_POP_WARNING
