@@ -345,15 +345,9 @@ void ChannelProcessor<KeyType>::close() && {
 
 template <typename KeyType>
 ChannelProcessor<KeyType> createChannelProcessor(
-    folly::Executor::KeepAlive<> executor,
-    std::shared_ptr<RateLimiter> rateLimiter,
-    size_t numSequencedExecutors) {
-  CHECK_GT(numSequencedExecutors, 0);
-  auto executors =
-      std::vector<folly::Executor::KeepAlive<folly::SequencedExecutor>>();
-  for (size_t i = 0; i < numSequencedExecutors; i++) {
-    executors.push_back(folly::SerialExecutor::create(executor));
-  }
+    std::vector<folly::Executor::KeepAlive<folly::SequencedExecutor>> executors,
+    std::shared_ptr<RateLimiter> rateLimiter) {
+  CHECK_GT(executors.size(), 0);
   auto [mergeChannelReceiver, mergeChannel] =
       createMergeChannel<KeyType, Unit>(executors[0]);
   return ChannelProcessor<KeyType>(
@@ -363,5 +357,20 @@ ChannelProcessor<KeyType> createChannelProcessor(
           std::move(mergeChannel),
           std::move(mergeChannelReceiver)));
 }
+
+template <typename KeyType>
+ChannelProcessor<KeyType> createChannelProcessor(
+    folly::Executor::KeepAlive<> executor,
+    std::shared_ptr<RateLimiter> rateLimiter,
+    size_t numSequencedExecutors) {
+  CHECK_GT(numSequencedExecutors, 0);
+  auto executors =
+      std::vector<folly::Executor::KeepAlive<folly::SequencedExecutor>>();
+  for (size_t i = 0; i < numSequencedExecutors; i++) {
+    executors.push_back(folly::SerialExecutor::create(executor));
+  }
+  return createChannelProcessor<KeyType>(
+      std::move(executors), std::move(rateLimiter));
+}
 } // namespace channels
 } // namespace folly
