@@ -17,12 +17,15 @@
 #pragma once
 
 #include <memory>
+#include <unordered_map>
+#include <unordered_set>
 
 #include <folly/Function.h>
 #include <folly/ScopeGuard.h>
+#include <folly/SingletonThreadLocal.h>
 #include <folly/Synchronized.h>
-#include <folly/ThreadLocal.h>
 #include <folly/container/F14Map.h>
+#include <folly/synchronization/RelaxedAtomic.h>
 
 namespace folly {
 namespace fibers {
@@ -31,7 +34,7 @@ namespace detail {
 
 // ssize_t is a hash of FiberManager::Options
 template <typename EventBaseT>
-using Key = std::tuple<EventBaseT*, ssize_t, std::type_index>;
+using Key = std::tuple<EventBaseT*, ssize_t, const std::type_info*>;
 
 template <typename EventBaseT>
 Function<void()> makeOnEventBaseDestructionCallback(const Key<EventBaseT>& key);
@@ -39,6 +42,9 @@ Function<void()> makeOnEventBaseDestructionCallback(const Key<EventBaseT>& key);
 template <typename EventBaseT>
 class GlobalCache {
  public:
+  using TypeMap = std::unordered_map< //
+      std::type_index,
+      std::unordered_set<const std::type_info*>>;
   template <typename LocalT>
   static FiberManager& get(
       const Key<EventBaseT>& key,
@@ -51,6 +57,8 @@ class GlobalCache {
     return instance().eraseImpl(key);
   }
 
+  static TypeMap getTypeMap() { return instance().getTypeMapImpl(); }
+
  private:
   GlobalCache() = default;
 
@@ -73,9 +81,13 @@ class GlobalCache {
       }
     };
 
+    auto mkey = MapKey(std::get<0>(key), std::get<1>(key), *std::get<2>(key));
+
     std::lock_guard<std::mutex> lg(mutex_);
 
-    auto& fmPtrRef = map_[key];
+    types_[std::get<2>(mkey)].insert(std::get<2>(key));
+
+    auto& fmPtrRef = map_[mkey];
 
     if (!fmPtrRef) {
       constructed = true;
@@ -89,17 +101,28 @@ class GlobalCache {
   }
 
   std::unique_ptr<FiberManager> eraseImpl(const Key<EventBaseT>& key) {
+    auto mkey = MapKey(std::get<0>(key), std::get<1>(key), *std::get<2>(key));
+
     std::lock_guard<std::mutex> lg(mutex_);
 
-    DCHECK_EQ(map_.count(key), 1u);
+    DCHECK_EQ(map_.count(mkey), 1u);
 
-    auto ret = std::move(map_[key]);
-    map_.erase(key);
+    auto ret = std::move(map_[mkey]);
+    map_.erase(mkey);
     return ret;
   }
 
+  TypeMap getTypeMapImpl() {
+    std::lock_guard<std::mutex> lg(mutex_);
+
+    return types_;
+  }
+
+  using MapKey = std::tuple<EventBaseT*, ssize_t, std::type_index>;
+
   std::mutex mutex_;
-  folly::F14NodeMap<Key<EventBaseT>, std::unique_ptr<FiberManager>> map_;
+  std::unordered_map<MapKey, std::unique_ptr<FiberManager>> map_;
+  TypeMap types_; // can have multiple type_info obj's for one type_index
 };
 
 constexpr size_t kEraseListMaxSize = 64;
@@ -110,11 +133,11 @@ class ThreadLocalCache {
   template <typename LocalT>
   static FiberManager& get(
       uint64_t token, EventBaseT& evb, const FiberManager::Options& opts) {
-    return instance()->template getImpl<LocalT>(token, evb, opts);
+    return STL::get().template getImpl<LocalT>(token, evb, opts);
   }
 
   static void erase(const Key<EventBaseT>& key) {
-    for (auto& localInstance : instance().accessAllThreads()) {
+    for (auto& localInstance : STL::accessAllThreads()) {
       localInstance.eraseInfo_.withWLock([&](auto& info) {
         if (info.eraseList.size() >= kEraseListMaxSize) {
           info.eraseAll = true;
@@ -127,47 +150,51 @@ class ThreadLocalCache {
   }
 
  private:
+  struct TLTag {};
+  template <typename Base>
+  struct Derived : Base {
+    using Base::Base;
+  };
+  using STL = SingletonThreadLocal<Derived<ThreadLocalCache>, TLTag>;
+
   ThreadLocalCache() = default;
 
-  struct ThreadLocalCacheTag {};
-  using ThreadThreadLocalCache =
-      ThreadLocal<ThreadLocalCache, ThreadLocalCacheTag>;
-
-  // Leak this intentionally. During shutdown, we may call getFiberManager,
-  // and want access to the fiber managers during that time.
-  static ThreadThreadLocalCache& instance() {
-    static auto ret =
-        new ThreadThreadLocalCache([]() { return new ThreadLocalCache(); });
-    return *ret;
-  }
-
   template <typename LocalT>
   FiberManager& getImpl(
       uint64_t token, EventBaseT& evb, const FiberManager::Options& opts) {
-    eraseImpl();
-
-    auto key = make_tuple(&evb, token, std::type_index(typeid(LocalT)));
-    auto& fmPtrRef = map_[key];
-    if (!fmPtrRef) {
-      fmPtrRef = &GlobalCache<EventBaseT>::template get<LocalT>(key, evb, opts);
+    if (eraseRequested_) {
+      eraseImpl();
     }
 
-    DCHECK(fmPtrRef != nullptr);
+    auto key = Key<EventBaseT>(&evb, token, &typeid(LocalT));
+    auto it = map_.find(key);
+    if (it != map_.end()) {
+      DCHECK(it->second != nullptr);
+      return *it->second;
+    }
 
-    return *fmPtrRef;
+    return getSlowImpl<LocalT>(key, evb, opts);
   }
 
-  void eraseImpl() {
-    if (!eraseRequested_.load()) {
-      return;
-    }
+  template <typename LocalT>
+  FOLLY_NOINLINE FiberManager& getSlowImpl(
+      Key<EventBaseT> key, EventBaseT& evb, const FiberManager::Options& opts) {
+    auto& ref = GlobalCache<EventBaseT>::template get<LocalT>(key, evb, opts);
+    map_.emplace(key, &ref);
+    return ref;
+  }
 
+  FOLLY_NOINLINE void eraseImpl() {
+    auto types = GlobalCache<EventBaseT>::getTypeMap(); // big copy!!!
     eraseInfo_.withWLock([&](auto& info) {
       if (info.eraseAll) {
         map_.clear();
       } else {
         for (auto& key : info.eraseList) {
-          map_.erase(key);
+          for (auto type : types[*std::get<2>(key)]) {
+            map_.erase( // can have multiple type_info obj's for one type_index
+                std::make_tuple(std::get<0>(key), std::get<1>(key), type));
+          }
         }
       }
 
@@ -177,8 +204,8 @@ class ThreadLocalCache {
     });
   }
 
-  folly::F14NodeMap<Key<EventBaseT>, FiberManager*> map_;
-  std::atomic<bool> eraseRequested_{false};
+  folly::F14FastMap<Key<EventBaseT>, FiberManager*> map_;
+  relaxed_atomic<bool> eraseRequested_{false};
 
   struct EraseInfo {
     bool eraseAll{false};
