@@ -63,7 +63,7 @@ class AsyncSSLSocketConnector;
  * this connection.  For client connections, the timeout should not be
  * active if there are no requests pending on the connection.
  * Additionally, if a client has multiple pending requests, it will
- * ususally want a separate timeout for each request, rather than a
+ * usually want a separate timeout for each request, rather than a
  * single read timeout.
  *
  * The write API is fairly intuitive: a user can request to send a
@@ -366,8 +366,8 @@ class AsyncSSLSocket : public AsyncSocket {
    * error. In that case, the READ/WRITE event should be registered,
    * and a flag (e.g., writeBlockedOnRead) should be set to indiciate
    * the condition. In the next invocation of read/write callback, if
-   * the flag is on, performWrite()/performRead() should be called in
-   * addition to the normal call to performRead()/performWrite(), and
+   * the flag is on, performWrite()/performReadMsg() should be called in
+   * addition to the normal call to performReadMsg()/performWrite(), and
    * the flag should be reset.
    */
 
@@ -393,6 +393,11 @@ class AsyncSSLSocket : public AsyncSocket {
     return "TLS";
   }
 
+  std::unique_ptr<folly::IOBuf> getExportedKeyingMaterial(
+      folly::StringPiece label,
+      std::unique_ptr<IOBuf> context,
+      uint16_t length) const override;
+
   void setEorTracking(bool track) override;
   size_t getRawBytesWritten() const override;
   size_t getRawBytesReceived() const override;
@@ -868,6 +873,8 @@ class AsyncSSLSocket : public AsyncSocket {
     return false;
   }
 
+  const char* getNegotiatedGroup() const;
+
  private:
   /**
    * Handle the return from invoking SSL_accept
@@ -876,6 +883,8 @@ class AsyncSSLSocket : public AsyncSocket {
 
   void init();
 
+  ReadResult performReadSingle(void* buf, const size_t buflen);
+
   // Need to clean this up during a cancel if callback hasn't fired yet.
   AsyncSSLSocketConnector* allocatedConnectCallback_;
 
@@ -905,13 +914,15 @@ class AsyncSSLSocket : public AsyncSocket {
   void handleInitialReadWrite() noexcept override {}
 
   WriteResult interpretSSLError(int rc, int error);
-  ReadResult performRead(void** buf, size_t* buflen, size_t* offset) override;
+  ReadResult performReadMsg(
+      struct ::msghdr&, AsyncReader::ReadCallback::ReadMode) override;
   WriteResult performWrite(
       const iovec* vec,
       uint32_t count,
       WriteFlags flags,
       uint32_t* countWritten,
-      uint32_t* partialWritten) override;
+      uint32_t* partialWritten,
+      WriteRequestTag writeTag) override;
 
   ssize_t performWriteIovec(
       const iovec* vec,
