@@ -21,6 +21,7 @@
 
 #include <chrono>
 #include <exception>
+#include <functional>
 #include <memory>
 #include <vector>
 #include <boost/variant.hpp>
@@ -73,6 +74,8 @@ namespace folly {
 class AsyncServerSocket : public DelayedDestruction, public AsyncSocketBase {
  public:
   typedef std::unique_ptr<AsyncServerSocket, Destructor> UniquePtr;
+  using CallbackAssignFunction =
+      std::function<int(AsyncServerSocket*, NetworkSocket)>;
   // Disallow copy, move, and default construction.
   AsyncServerSocket(AsyncServerSocket&&) = delete;
 
@@ -358,6 +361,13 @@ class AsyncServerSocket : public DelayedDestruction, public AsyncSocketBase {
     }
   }
 
+  /**
+   * sets the callback assign function
+   */
+  void setCallbackAssignFunction(CallbackAssignFunction&& func) {
+    callbackAssignFunc_ = std::move(func);
+  }
+
   /**
    * enable zerocopy support for the server sockets -
    * the s = accept sockets inherit it
@@ -917,7 +927,13 @@ class AsyncServerSocket : public DelayedDestruction, public AsyncSocketBase {
   void enterBackoff();
   void backoffTimeoutExpired();
 
-  CallbackInfo* nextCallback() {
+  CallbackInfo* nextCallback(NetworkSocket socket = NetworkSocket()) {
+    if (callbackAssignFunc_ && socket != NetworkSocket()) {
+      auto num = callbackAssignFunc_(this, socket);
+      if (num >= 0) {
+        return &callbacks_[num % callbacks_.size()];
+      }
+    }
     CallbackInfo* info = &callbacks_[callbackIndex_];
 
     ++callbackIndex_;
@@ -985,6 +1001,7 @@ class AsyncServerSocket : public DelayedDestruction, public AsyncSocketBase {
   uint32_t callbackIndex_;
   BackoffTimeout* backoffTimeout_;
   std::vector<CallbackInfo> callbacks_;
+  CallbackAssignFunction callbackAssignFunc_;
   bool keepAliveEnabled_;
   bool reusePortEnabled_{false};
   bool closeOnExec_;
