@@ -408,6 +408,19 @@ class AsyncWriter {
    public:
     virtual ~WriteCallback() = default;
 
+    /**
+     * writeStarting() will be invoked right before bytes are written to the
+     * socket.
+     *
+     * This enables the callback implementation to determine the raw (socket)
+     * byte offset for the first byte in this write's buffer. This may be
+     * different than the number of bytes written at the application layer in
+     * the case of TLS and other transformations.
+     *
+     * Intermediary transport layers should forward this signal.
+     */
+    virtual void writeStarting() noexcept {}
+
     /**
      * writeSuccess() will be invoked when all of the data has been
      * successfully written.
@@ -1239,9 +1252,9 @@ class AsyncTransport : public DelayedDestruction,
    *
    * @param observer     Observer to add (implements LifecycleObserver).
    */
-  virtual void addLifecycleObserver(LifecycleObserver* /* observer */) {
-    folly::terminate_with<std::runtime_error>(
-        "addLifecycleObserver() not supported");
+  virtual void addLifecycleObserver(LifecycleObserver*) {
+    // A LifecycleObserver should not depend on receiving byteEventsUnavailable
+    // in this case.
   }
 
   /**
@@ -1251,8 +1264,7 @@ class AsyncTransport : public DelayedDestruction,
    * @return             Whether observer found and removed from list.
    */
   virtual bool removeLifecycleObserver(LifecycleObserver* /* observer */) {
-    folly::terminate_with<std::runtime_error>(
-        "removeLifecycleObserver() not supported");
+    return false;
   }
 
   /**
@@ -1262,8 +1274,7 @@ class AsyncTransport : public DelayedDestruction,
    */
   FOLLY_NODISCARD virtual std::vector<LifecycleObserver*>
   getLifecycleObservers() const {
-    folly::terminate_with<std::runtime_error>(
-        "getLifecycleObservers() not supported");
+    return std::vector<LifecycleObserver*>();
   }
 
   /**
