@@ -169,6 +169,19 @@ class DecoratedAsyncTransportWrapper : public folly::AsyncTransport {
     transport_->setReplaySafetyCallback(callback);
   }
 
+  void addLifecycleObserver(LifecycleObserver* observer) override {
+    transport_->addLifecycleObserver(observer);
+  }
+
+  bool removeLifecycleObserver(LifecycleObserver* observer) override {
+    return transport_->removeLifecycleObserver(observer);
+  }
+
+  FOLLY_NODISCARD std::vector<LifecycleObserver*> getLifecycleObservers()
+      const override {
+    return transport_->getLifecycleObservers();
+  }
+
   const AsyncTransportCertificate* getPeerCertificate() const override {
     return transport_->getPeerCertificate();
   }
@@ -195,6 +208,11 @@ class DecoratedAsyncTransportWrapper : public folly::AsyncTransport {
     transport_->setZeroCopyEnableFunc(func);
   }
 
+  AsyncTransport::UniquePtr tryExchangeWrappedTransport(
+      AsyncTransport::UniquePtr& transport) override {
+    return std::exchange(transport_, std::move(transport));
+  }
+
  protected:
   ~DecoratedAsyncTransportWrapper() override {}
 
