@@ -435,7 +435,7 @@ bool EventBase::loopMain(int flags, bool ignoreKeepAlive) {
       maxLatencyLoopTime_.addSample(loop_time, busy);
 
       if (observer_) {
-        if (observerSampleCount_++ == observer_->getSampleRate()) {
+        if (++observerSampleCount_ >= observer_->getSampleRate()) {
           observerSampleCount_ = 0;
           observer_->loopSample(busy.count(), idle.count());
         }
@@ -875,6 +875,10 @@ const std::string& EventBase::getName() {
   return name_;
 }
 
+std::thread::id EventBase::getLoopThreadId() {
+  return loopThread_.load(std::memory_order_relaxed);
+}
+
 void EventBase::scheduleAt(Func&& fn, TimePoint const& timeout) {
   auto duration = timeout - now();
   timer().scheduleTimeoutFn(
