@@ -818,12 +818,6 @@ std::string SSLContext::getErrors(int errnoCopy) {
   return errors;
 }
 
-void SSLContext::enableTLS13() {
-#if FOLLY_OPENSSL_PREREQ(1, 1, 0)
-  SSL_CTX_set_max_proto_version(ctx_, 0);
-#endif
-}
-
 void SSLContext::disableTLS13() {
 #if FOLLY_OPENSSL_PREREQ(1, 1, 0)
   SSL_CTX_set_max_proto_version(ctx_, TLS1_2_VERSION);
