@@ -77,13 +77,17 @@ class SendMsgParamsCallbackBase
   void getAncillaryData(
       folly::WriteFlags flags,
       void* data,
+      const AsyncSocket::WriteRequestTag& writeTag,
       const bool byteEventsEnabled) noexcept override {
-    oldCallback_->getAncillaryData(flags, data, byteEventsEnabled);
+    oldCallback_->getAncillaryData(flags, data, writeTag, byteEventsEnabled);
   }
 
   uint32_t getAncillaryDataSize(
-      folly::WriteFlags flags, const bool byteEventsEnabled) noexcept override {
-    return oldCallback_->getAncillaryDataSize(flags, byteEventsEnabled);
+      folly::WriteFlags flags,
+      const AsyncSocket::WriteRequestTag& writeTag,
+      const bool byteEventsEnabled) noexcept override {
+    return oldCallback_->getAncillaryDataSize(
+        flags, writeTag, byteEventsEnabled);
   }
 
   std::shared_ptr<AsyncSSLSocket> socket_;
@@ -126,6 +130,7 @@ class SendMsgAncillaryDataCallback : public SendMsgParamsCallbackBase {
   void getAncillaryData(
       folly::WriteFlags flags,
       void* data,
+      const AsyncSocket::WriteRequestTag& writeTag,
       const bool byteEventsEnabled) noexcept override {
     // getAncillaryData is called through a long chain of functions after send
     // record the observed write flags so we can compare later
@@ -135,17 +140,20 @@ class SendMsgAncillaryDataCallback : public SendMsgParamsCallbackBase {
       std::cerr << "getAncillaryData: copying data" << std::endl;
       memcpy(data, ancillaryData_.data(), ancillaryData_.size());
     } else {
-      oldCallback_->getAncillaryData(flags, data, byteEventsEnabled);
+      oldCallback_->getAncillaryData(flags, data, writeTag, byteEventsEnabled);
     }
   }
 
   uint32_t getAncillaryDataSize(
-      folly::WriteFlags flags, const bool byteEventsEnabled) noexcept override {
+      folly::WriteFlags flags,
+      const AsyncSocket::WriteRequestTag& writeTag,
+      const bool byteEventsEnabled) noexcept override {
     if (ancillaryData_.size()) {
       std::cerr << "getAncillaryDataSize: returning size" << std::endl;
       return ancillaryData_.size();
     } else {
-      return oldCallback_->getAncillaryDataSize(flags, byteEventsEnabled);
+      return oldCallback_->getAncillaryDataSize(
+          flags, writeTag, byteEventsEnabled);
     }
   }
 
