@@ -72,6 +72,11 @@ class MockAsyncTransport : public AsyncTransport {
   MOCK_METHOD(bool, isReplaySafe, (), (const));
   MOCK_METHOD(
       void, setReplaySafetyCallback, (AsyncTransport::ReplaySafetyCallback*));
+  MOCK_METHOD(void, addLifecycleObserver, (AsyncTransport::LifecycleObserver*));
+  MOCK_METHOD(
+      bool, removeLifecycleObserver, (AsyncTransport::LifecycleObserver*));
+  MOCK_METHOD(
+      std::vector<LifecycleObserver*>, getLifecycleObservers, (), (const));
   MOCK_METHOD(std::string, getSecurityProtocol, (), (const));
   MOCK_METHOD(
       const AsyncTransportCertificate*, getPeerCertificate, (), (const));
