@@ -78,8 +78,7 @@ void testAllocSize(uint32_t requestedCapacity) {
 }
 
 TEST(IOBuf, AllocSizes) {
-  // cover small evil values exhaustively, including the
-  // kDefaultCombinedBufSize transition.
+  // Cover small evil values exhaustively.
   for (uint32_t i = 0; i < 1234; i++) {
     testAllocSize(i);
   }
@@ -263,12 +262,12 @@ TEST(IOBuf, WrapBuffer) {
   if (folly::kIsSanitizeAddress) {
     const uint32_t size5 = 100;
     uint8_t buf5[size5];
-    EXPECT_DEATH(IOBuf::wrapBuffer(buf5, size5 + 1), "asan_region_is_poisoned");
+    EXPECT_DEATH(IOBuf::wrapBuffer(buf5, size5 + 1), "stack-buffer-overflow");
 
     const uint32_t size6 = 100;
     std::vector<uint8_t> buf6(size6);
     EXPECT_DEATH(
-        IOBuf::wrapBuffer(buf6.data(), size6 + 1), "asan_region_is_poisoned");
+        IOBuf::wrapBuffer(buf6.data(), size6 + 1), "heap-buffer-overflow");
   }
 }
 
@@ -893,7 +892,14 @@ TEST(IOBuf, takeOwnershipUniquePtr) {
 TEST(IOBuf, Alignment) {
   size_t alignment = alignof(std::max_align_t);
 
-  std::vector<size_t> sizes{0, 1, 64, 256, 1024, 1 << 10};
+  std::vector<size_t> sizes;
+  sizes.reserve(259);
+  for (size_t i = 0; i <= 256; i++) {
+    sizes.push_back(i);
+  }
+  sizes.push_back(1024);
+  sizes.push_back(1 << 10);
+
   for (size_t size : sizes) {
     auto buf = IOBuf::create(size);
     uintptr_t p = reinterpret_cast<uintptr_t>(buf->data());
