@@ -33,7 +33,7 @@
           __folly_detail_safe_assert_arg{                                 \
               u ? nullptr : #expr,                                        \
               __FILE__,                                                   \
-              __LINE__,                                                   \
+              FOLLY_PP_CONSTINIT_LINE_UNSIGNED,                           \
               __folly_detail_safe_assert_fun,                             \
               ::folly::detail::safe_assert_msg_types<                     \
                   decltype(::folly::detail::safe_assert_msg_types_seq_of( \
@@ -140,10 +140,13 @@ struct safe_assert_msg_types<safe_assert_msg_type_s<A...>> {
   using value_type = c_array<safe_assert_msg_type, sizeof...(A) + 1>;
   static constexpr value_type value = {{A..., safe_assert_msg_type::term}};
 };
+
+#if FOLLY_CPLUSPLUS < 201703L
 template <safe_assert_msg_type... A>
 constexpr
     typename safe_assert_msg_types<safe_assert_msg_type_s<A...>>::value_type
         safe_assert_msg_types<safe_assert_msg_type_s<A...>>::value;
+#endif
 
 struct safe_assert_arg {
   char const* expr;
