@@ -27,9 +27,10 @@
 #include <stdexcept>
 #include <tuple>
 
-#include <boost/lexical_cast.hpp>
+#include <fmt/format.h>
 #include <glog/logging.h>
 
+#include <folly/Random.h>
 #include <folly/container/Foreach.h>
 #include <folly/portability/GTest.h>
 
@@ -177,6 +178,35 @@ void test128Bit2String() {
   EXPECT_EQ(to<Sint>(to<String>(svalue)), svalue);
   value = numeric_limits<Sint>::max();
   EXPECT_EQ(to<Sint>(to<String>(svalue)), svalue);
+
+  {
+    Uint v = 1;
+    for (size_t zeros = 0; zeros < 39; ++zeros, v *= 10) {
+      EXPECT_EQ(to<String>(v), String("1") + String(zeros, '0'));
+    }
+  }
+
+  // Compare the results with fmt on random values. If they disagree, either
+  // to() or fmt is incorrect.
+  for (size_t b = 0; b <= 64; ++b) {
+    for (size_t i = 0; i < 1000; ++i) {
+      const auto low = folly::Random::rand64();
+      if (b < 64) {
+        const auto high = folly::Random::rand64(uint64_t(1) << b);
+        const auto v =
+            (folly::Random::oneIn(2) ? -1 : 1) * ((Sint(high) << 64) | low);
+        const auto s = to<String>(v);
+        EXPECT_EQ(s, fmt::format("{}", v));
+        EXPECT_EQ(to<Sint>(s), v);
+      } else {
+        const auto high = folly::Random::rand64();
+        const auto v = (Uint(high) << 64) | low;
+        const auto s = to<String>(v);
+        EXPECT_EQ(s, fmt::format("{}", v));
+        EXPECT_EQ(to<Uint>(s), v);
+      }
+    }
+  }
 }
 
 #endif
@@ -672,6 +702,26 @@ TEST(Conv, DoubleToInt) {
   }
 }
 
+namespace {
+
+template <class From, class To>
+void testNotFiniteToInt() {
+  for (auto s : {"nan", "inf", "-inf"}) {
+    auto v = to<From>(s);
+    auto rv = folly::tryTo<To>(v);
+    EXPECT_FALSE(rv.hasValue()) << s << " " << rv.value();
+  }
+}
+
+} // namespace
+
+TEST(Conv, NotFiniteToInt) {
+  testNotFiniteToInt<float, int64_t>();
+  testNotFiniteToInt<float, uint64_t>();
+  testNotFiniteToInt<double, int64_t>();
+  testNotFiniteToInt<double, uint64_t>();
+}
+
 TEST(Conv, EnumToInt) {
   enum A { x = 42, y = 420, z = 65 };
   auto i = to<int>(x);
@@ -703,7 +753,7 @@ TEST(Conv, IntToEnum) {
   auto j = to<A>(100);
   EXPECT_EQ(j, 100);
   try {
-    auto i2 = to<A>(5000000000L);
+    auto i2 = to<A>(5000000000LL);
     LOG(ERROR) << "to<A> returned " << static_cast<unsigned int>(i2)
                << " instead of throwing";
     ADD_FAILURE();
@@ -843,6 +893,7 @@ TEST(Conv, FloatToInt) {
   EXPECT_THROW(
       to<uint64_t>(static_cast<float>(std::numeric_limits<uint64_t>::max())),
       std::range_error);
+  EXPECT_THROW(to<uint64_t>(static_cast<float>(-1)), std::range_error);
 }
 
 TEST(Conv, IntToFloat) {
