@@ -484,17 +484,19 @@ class TestExtendingFormatter
             Args...>(str, std::forward<Args>(args)...) {}
 
   template <size_t K, class Callback>
-  void doFormatArg(FormatArg& arg, Callback& cb) const {
+  static void doFormatArg(
+      const detail::BaseFormatterBase& obj, FormatArg& arg, Callback& cb) {
     std::string result;
     auto appender = [&result](StringPiece s) {
       result.append(s.data(), s.size());
     };
-    this->template getFormatValue<K>().format(arg, appender);
+    auto& self = static_cast<const TestExtendingFormatter&>(obj);
+    self.template getFormatValue<K>().format(arg, appender);
     result = sformat("{{{}}}", result);
     cb(StringPiece(result));
   }
 
-  friend class BaseFormatter<
+  friend BaseFormatter<
       TestExtendingFormatter<containerMode, Args...>,
       containerMode,
       Args...>;
