@@ -37,17 +37,6 @@ static constexpr std::size_t kTooBig = folly::constexpr_max(
     std::size_t{std::numeric_limits<uint32_t>::max()},
     std::size_t{1} << (8 * sizeof(std::size_t) - 14));
 
-TEST(allocateBytes, simple) {
-  auto p = allocateBytes(10);
-  EXPECT_TRUE(p != nullptr);
-  deallocateBytes(p, 10);
-}
-
-TEST(allocateBytes, zero) {
-  auto p = allocateBytes(0);
-  deallocateBytes(p, 0);
-}
-
 TEST(operatorNewDelete, zero) {
   auto p = ::operator new(0);
   EXPECT_TRUE(p != nullptr);
@@ -174,6 +163,17 @@ TEST(copy_to_shared_ptr, example) {
   EXPECT_EQ(17, *s);
 }
 
+TEST(copy_through_unique_ptr, example) {
+  std::unique_ptr<int> p = std::make_unique<int>(17);
+  std::unique_ptr<int> s = copy_through_unique_ptr(p);
+  EXPECT_EQ(17, *s);
+  EXPECT_EQ(17, *p);
+
+  p.reset();
+  s = copy_through_unique_ptr(p);
+  EXPECT_EQ(s, nullptr);
+}
+
 TEST(to_erased_unique_ptr, example) {
   erased_unique_ptr ptr = empty_erased_unique_ptr();
 
