@@ -16,6 +16,7 @@
 
 #include <folly/Singleton.h>
 
+#include <cstdlib>
 #include <thread>
 
 #include <boost/thread/barrier.hpp>
@@ -26,10 +27,7 @@
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 #include <folly/test/SingletonTestStructs.h>
-
-#ifndef _MSC_VER
-#include <folly/Subprocess.h>
-#endif
+#include <folly/test/TestUtils.h>
 
 FOLLY_GNU_DISABLE_WARNING("-Wdeprecated-declarations")
 
@@ -221,30 +219,42 @@ using SingletonNaughtyUsage2 = Singleton<T, Tag, NaughtyUsageTag2>;
 TEST(Singleton, NaughtyUsage) {
   auto& vault = *SingletonVault::singleton<NaughtyUsageTag>();
 
+  const char* kBeforeRegistrationCompleteMsg =
+      ".*Singleton .* requested before registrationComplete().*";
+  SingletonNaughtyUsage2<int> s0;
+  EXPECT_DEATH(s0.try_get(), kBeforeRegistrationCompleteMsg);
+
   vault.registrationComplete();
 
-  // Unregistered.
-  EXPECT_DEATH(Singleton<Watchdog>::try_get(), "");
-  EXPECT_DEATH(Singleton<Watchdog>::apply([](auto* v) { return v; }), "");
-  EXPECT_DEATH(SingletonNaughtyUsage<Watchdog>::try_get(), "");
+  const char* kUnregisteredMsg =
+      ".*Creating instance for unregistered singleton.*";
+  EXPECT_DEATH(Singleton<Watchdog>::try_get(), kUnregisteredMsg);
   EXPECT_DEATH(
-      SingletonNaughtyUsage<Watchdog>::apply([](auto* v) { return v; }), "");
+      Singleton<Watchdog>::apply([](auto* v) { return v; }), kUnregisteredMsg);
+  EXPECT_DEATH(SingletonNaughtyUsage<Watchdog>::try_get(), kUnregisteredMsg);
+  EXPECT_DEATH(
+      SingletonNaughtyUsage<Watchdog>::apply([](auto* v) { return v; }),
+      kUnregisteredMsg);
 
   vault.destroyInstances();
 
   auto& vault2 = *SingletonVault::singleton<NaughtyUsageTag2>();
 
-  EXPECT_DEATH(SingletonNaughtyUsage2<Watchdog>::try_get(), "");
+  EXPECT_DEATH(SingletonNaughtyUsage2<Watchdog>::try_get(), kUnregisteredMsg);
   EXPECT_DEATH(
-      SingletonNaughtyUsage2<Watchdog>::apply([](auto* v) { return v; }), "");
+      SingletonNaughtyUsage2<Watchdog>::apply([](auto* v) { return v; }),
+      kUnregisteredMsg);
   SingletonNaughtyUsage2<Watchdog> watchdog_singleton;
 
-  // double registration
-  EXPECT_DEATH([]() { SingletonNaughtyUsage2<Watchdog> w2; }(), "");
+  const char* kDoubleRegistration =
+      "Double registration of singletons of the same underlying type";
+  EXPECT_DEATH(
+      [] { SingletonNaughtyUsage2<Watchdog> w2; }(), kDoubleRegistration);
   vault2.destroyInstances();
 
-  // double registration after destroy
-  EXPECT_DEATH([]() { SingletonNaughtyUsage2<Watchdog> w3; }(), "");
+  // Double registration after destroy.
+  EXPECT_DEATH(
+      [] { SingletonNaughtyUsage2<Watchdog> w3; }(), kDoubleRegistration);
 }
 
 struct SharedPtrUsageTag {};
@@ -752,26 +762,6 @@ TEST(Singleton, MockTestWithApply) {
   vault.destroyInstances();
 }
 
-#ifndef _MSC_VER
-// Subprocess isn't currently supported under MSVC.
-TEST(Singleton, DoubleRegistrationLogging) {
-  const auto basename = "singleton_double_registration";
-  const auto sub = fs::executable_path().remove_filename() / basename;
-  auto p = Subprocess(
-      std::vector<std::string>{sub.string()},
-      Subprocess::Options()
-          .stdinFd(Subprocess::DEV_NULL)
-          .stdoutFd(Subprocess::DEV_NULL)
-          .pipeStderr()
-          .closeOtherFds());
-  auto err = p.communicate("").second;
-  auto res = p.wait();
-  EXPECT_EQ(ProcessReturnCode::KILLED, res.state());
-  EXPECT_EQ(SIGABRT, res.killSignal());
-  EXPECT_THAT(err, testing::StartsWith("Double registration of singletons"));
-}
-#endif
-
 // Singleton using a non default constructor test/example:
 struct X {
   X() : X(-1, "unset") {}
@@ -1024,6 +1014,10 @@ TEST(Singleton, LeakySingletonTSAN) {
 }
 
 TEST(Singleton, ShutdownTimer) {
+  // TSAN will SIGSEGV if the shutdown timer activates (it spawns a new thread,
+  // which TSAN doesn't like).
+  SKIP_IF(folly::kIsSanitizeThread);
+
   struct VaultTag {};
   struct PrivateTag {};
   struct Object {
