@@ -17,6 +17,7 @@
 #include <folly/Traits.h>
 
 #include <cstring>
+#include <memory>
 #include <string>
 #include <type_traits>
 #include <utility>
@@ -553,3 +554,17 @@ TEST(Traits, type_pack_element_t) {
   EXPECT_TRUE((is_detected_v<test::native_ic, index_constant<0>, int>));
   EXPECT_FALSE((is_detected_v<test::native_ic, index_constant<0>>));
 }
+
+TEST(Traits, is_allocator) {
+  static_assert(is_allocator_v<std::allocator<int>>, "");
+  static_assert(is_allocator<std::allocator<int>>::value, "");
+
+  static_assert(is_allocator_v<std::allocator<std::string>>, "");
+  static_assert(is_allocator<std::allocator<std::string>>::value, "");
+
+  static_assert(!is_allocator_v<int>, "");
+  static_assert(!is_allocator<int>::value, "");
+
+  static_assert(!is_allocator_v<std::string>, "");
+  static_assert(!is_allocator<std::string>::value, "");
+}
