@@ -21,6 +21,7 @@
 #include <mutex>
 
 #include <glog/logging.h>
+#include <glog/raw_logging.h>
 
 #include <folly/Likely.h>
 #include <folly/lang/Hint.h>
@@ -57,9 +58,10 @@ static void ensureAsyncRootTlsKeyIsInitialised() noexcept {
   (void)pthread_once(&initialiseTlsKeyFlag, []() noexcept {
     int result = pthread_key_create(&folly_async_stack_root_tls_key, nullptr);
     if (UNLIKELY(result != 0)) {
-      LOG(FATAL)
-          << "Failed to initialise folly_async_stack_root_tls_key: (error:"
-          << result << ")";
+      RAW_LOG(
+          FATAL,
+          "Failed to initialise folly_async_stack_root_tls_key: (error: %d)",
+          result);
       std::terminate();
     }
   });
@@ -74,7 +76,10 @@ struct AsyncStackRootHolder {
     const int result =
         pthread_setspecific(folly_async_stack_root_tls_key, this);
     if (FOLLY_UNLIKELY(result != 0)) {
-      LOG(FATAL) << "Failed to set current thread's AsyncStackRoot";
+      RAW_LOG(
+          FATAL,
+          "Failed to set current thread's AsyncStackRoot: (error: %d)",
+          result);
       std::terminate();
     }
   }
